/////////////////////////////////////////////////////////////////////////////
// This file is a part of the OpenRISC 1601 Verilog-HDL model
// Copyright (C) 2000 Jimmy C. Chen and OPENCORES.ORG
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// See the file COPYING for the full details of the license.
//
// OPENCORES.ORG is looking for new open source IP cores and developers that
// would like to help in our mission.
////////////////////////////////////////////////////////////////////////////
// IP core: OpenRISC 1601, a 16-bit instruction set/32-bit datapath RISC 
//          processor core of OR1K family, according to OR16 ISA.
// Features:
//  * fast exception processing (see architecture manual for details)
//  * yet optimized for size (without MMU and Caches)
// 
// Documentation: http://www.opencores.org/cores/or1k/or1601/
////////////////////////////////////////////////////////////////////////////
// File: BPU.v
// Author: Jimmy Chen-Min Chen, jimmy@ee.nctu.edu.tw, 06/18/2000
// Description: Branch Processing Unit
//  
// Architecture Notes:
// 	BPU must generate the final condition of the conditional branch. 
//  
//  
// TO DO: branch prediction (always-taken or one-bit branch prediction)
//
////////////////////////////////////////////////////////////////////////////
// Revision History
// $Log$
`include "pool/timescale.v"
`include "pool/sys.h"
`include "pool/or1601isa.h"

module BPU(
	STALL_BPU,

	iBF_D,
	iBNF_D,

	BF_MR,
	BNF_MR,
	BF_ER,
	BNF_ER,
	
	TEST_ER,
	SFU_ER,
	SFS_ER,
        iFLAG_E,
        FLAG_MR,
        FLAG_WR,

	iMISS_PREDICT_D,
	iMISS_PREDICT_E,

	CLK,
	RSTN
);
	input		iBF_D, iBNF_D;

	input  		BF_MR, BNF_MR;
	input  		BF_ER, BNF_ER;
	input		TEST_ER, SFU_ER, SFS_ER;
	input		iFLAG_E, FLAG_MR, FLAG_WR;
	output		iMISS_PREDICT_D, iMISS_PREDICT_E;
	input		STALL_BPU;
	input		CLK, RSTN;

	reg   		iMISS_PREDICT_D, iMISS_PREDICT_E;

always @(iBF_D or iBNF_D or BF_MR or BNF_MR or FLAG_MR or iFLAG_E or
	 BF_ER or BNF_ER or SFS_ER or SFU_ER or TEST_ER
) begin
	if( (SFS_ER || SFU_ER || TEST_ER) && (iBF_D && ~iFLAG_E || iBNF_D && iFLAG_E) ||
	   ~(SFS_ER || SFU_ER || TEST_ER) && (iBF_D && ~FLAG_MR || iBNF_D && FLAG_MR)
	  )
		iMISS_PREDICT_D = 1'b1;
	else
		iMISS_PREDICT_D = 1'b0;


	if(BF_ER && ~FLAG_MR || BNF_ER && FLAG_MR)
		iMISS_PREDICT_E = 1'b1;
	else
		iMISS_PREDICT_E = 1'b0;
end


endmodule
