
`include "pool/timescale.v"
`include "pool/sys.h"

module CPU(
// 
        INTREQN,                // interrupt request low active

// OR1K native bus interface(NBI)
        NBI_ADDR,               // addr out
        NBI_DO,                 // data out
        NBI_SIZE,               // 2 bit size
        NBI_RW,                 // read(1)/write(0)
        NBI_BURSTN,             // burst transaction
        NBI_REQN,               // NBI request to bridge

        NBI_DI,                 // data in
        NBI_GNTN,               // bridge grants the request of NBI_REQN

// system
        RSTN,
        CLK
);

	input  [2:0]	INTREQN;
	output [31:0]	NBI_ADDR;
	output [31:0]	NBI_DO;
	output [1:0]	NBI_SIZE;
	output		NBI_RW, NBI_BURSTN, NBI_REQN;

	input  [31:0]	NBI_DI;
	input		NBI_GNTN;

	input		RSTN, CLK;
// internal signals
	wire   [31:0]	INU_ADDR;	
	wire   [31:0]	INU_DATA;	
	wire   [31:0]	LSU_ADDR;	
	wire   [31:0]	iLSU_ADDR_MR;	
	wire   [31:0]	LSU_DI;
	wire   [31:0]	LSU_DO;
	wire   [1:0]	LSU_SIZE;

BIU BIU1(
	.INTREQN	(INTREQN),		// interrupt request low active

// IU interface
        .INU_ADDR	({INU_ADDR[31:2],2'b0}),
        .INU_DATA	(INU_DATA),
	.INU_UC		(1'b1),			// insn in uncached region
        .INU_REQ	(INU_REQ),

	.iALL_UPDATE_IFA_W	(iALL_UPDATE_IFA_W),

        .iBIU2INU_BUSY	(iBIU2INU_BUSY),	// BIU tells INU that she's busy, or bridge is busy

        .iLSU_ADDR_MR	(iLSU_ADDR_MR),
        .LSU_ADDR	(LSU_ADDR),
        .LSU_DI		(LSU_DI),		// LSU's data input
        .LSU_DO		(LSU_DO),			
        .LSU_STORREQ	(LSU_STORREQ),
        .LSU_SIZE	(LSU_SIZE),
        .LSU_LOADREQ	(LSU_LOADREQ),
	.LSU_UC		(1'b1),			// LSU uncached region

        .iBIU2LSU_BUSY	(iBIU2LSU_BUSY),	// BIU tells LSU that she's busy, or bridge is busy 

// OR1K native bus interface(NBI)
	.NBI_ADDR	(NBI_ADDR),		// addr out
	.NBI_DO		(NBI_DO),		// data out
	.NBI_SIZE	(NBI_SIZE),		// 2 bit size
	.NBI_RW		(NBI_RW),		// read(1)/write(0)
	.NBI_BURSTN	(NBI_BURSTN),		// burst transaction
	.NBI_REQN	(NBI_REQN),		// NBI request to bridge

	.NBI_DI		(NBI_DI),		// data in
	.NBI_GNTN	(NBI_GNTN),		// bridge grants the request of NBI_REQN

// system
        .RSTN		(RSTN),
        .CLK		(CLK)
);

	reg		RSTN_D1, RSTN_D2;

	always @(posedge CLK) begin
		RSTN_D1 <= #(`UD) RSTN;
		RSTN_D2 <= #(`UD) RSTN_D1;
	end

IU IU1(
	.ASYNC_EXT_INTN	(INTREQN),		// asynchronous external interrupt pins, low active

	.INU_ADDR	(INU_ADDR), 
	.INU_DATA	(INU_DATA), 
	.INU_REQ	(INU_REQ), 
	.iBIU2INU_BUSY	(iBIU2INU_BUSY),
	.iALL_UPDATE_IFA_W	(iALL_UPDATE_IFA_W),
    
    	.iLSU_ADDR_MR	(iLSU_ADDR_MR), 
    	.LSU_ADDR	(LSU_ADDR), 
	.LSU_DI		(LSU_DI), 
	.LSU_DO		(LSU_DO), 
	.LSU_SIZE	(LSU_SIZE), 

	.LSU_LOADREQ	(LSU_LOADREQ), 
	.LSU_STORREQ	(LSU_STORREQ), 
	.BIU2LSU_BUSY	(iBIU2LSU_BUSY),
	.LSU_STALL	(),

	.RSTN_D2	(RSTN_D2), 
	.CLK		(CLK)
);

endmodule
