/////////////////////////////////////////////////////////////////////////////
// This file is a part of the OpenRISC 1601 Verilog-HDL model
// Copyright (C) 2000 Jimmy C. Chen and OPENCORES.ORG
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// See the file COPYING for the full details of the license.
//
// OPENCORES.ORG is looking for new open source IP cores and developers that
// would like to help in our mission.
////////////////////////////////////////////////////////////////////////////
// IP core: OpenRISC 1601, a 16-bit instruction set/32-bit datapath RISC 
//          processor core of OR1K family, according to OR16 ISA.
// Features:
//  * fast exception processing (see architecture manual for details)
//  * yet optimized for size (without MMU and Caches)
// 
// Documentation: http://www.opencores.org/cores/or1k/or1601/
////////////////////////////////////////////////////////////////////////////
// File: DEC.v
// Author: Jimmy Chen-Min Chen, jimmy@ee.nctu.edu.tw, 06/18/2000
// Description: decoding module
//  
// Architecture Notes:
// 	Decoding the instruction register.
// 	if you want to morph it to another ISA, modify this module. 
//  
// TO DO:
//
////////////////////////////////////////////////////////////////////////////
// Revision History
// $Log$
`include "pool/timescale.v"
`include "pool/sys.h"
`include "pool/or1601isa.h"

module DEC(
	STALL_DEC,
	EXT_PIPE_STALL,

	iEXCEPT_M,				// exception
	EXCEPT_BJDS_WR,				// exception in branch/jump delay slot
	RFE_MR,					// RFE clear SUPV bit in SYS_SR special register

	ASYNC_EXT_INTN,				// asynchronous external interrupt pins, low active
	INT_ENABLE_MR,
	//EXT_INT_MR,

        II_EXP_MR,                              // illegal insn exception(DEC)
        EXT_INT_MR,                             // external interrupt(DEC)
        SYS_MR,                                	// h.sys(DEC)
        BRK_MR,                                	// h.brk(DEC)

        iSPRG_RD_D,
        iSPRG_WR_D,
        SPRG_RD_ER,                        	// read enable at E stage
        SPRG_RD_MR,                        	// read enable at E stage
        SPRG_WR_ER,                        	// write enable

	PAIR_CNST_ER,               		// PAIR auto-increment constant, it may be shifted left 2bits, so total 6 bits(signed integer)
	LDST_PAIR_REQ_ER,

	INU_DATA,
	IR_DR,
	IR_ER,

	iIMME_CNST_D,
	iIALU_IMME_CONST,
	iSTOR_IMME_CONST,
	iLD_DELAY_SLOT_STALL,
	iMFSR_DELAY_SLOT_STALL,

//* decoding for branch
	JAL_ER,	
	BF_MR,
	BNF_MR,
	BF_ER,
	BNF_ER,

	JAL_JALR_MR,
	iJAL_JALR_D,

        iBF_D,
        iBNF_D,
        iJMP_D,
        iJAL_D,
        iJALR_JR_D,
        iBF_BNF_D,

//* register file read/write
	iRF_AADR, 
	iRF_BADR,
	iRF_AREN, 
	iRF_BREN,
	iRF_WADR_M,
	iRF_WEN_M,
	RF_WADR_ER,
	RF_WADR_MR,
	RF_WADR_WR,

	RF_WEN_ER,
	RF_WEN_MR,
	RF_WEN_WR,

//* decoding for LSU
	LSU_LOADREQ,
	iLOADREQ_D,
	iSTORREQ_D,
	iLDST_SZ_D,
	STORPAI_ER,
	STORPAI_MR,
	iLSU_SIGN_EXT_D,

//* decoding for ALU

	SFU_ER, 
	SFS_ER,

	iEXTOP, iEXTHS, iEXTHZ, iEXTBS, iEXTBZ,
	iMOVOP, iMOV, iMOVIBSE, iMOVIBZE, iMOVIBLS, iSWAPB, iSWAPH,
	iLOGOP, iXOR, iOR, iAND, iTEST, iCLR, iSET,
	iSHOP, iSHLA, iSHRA, iSHRL, iROR, iROL,
	iADDOP, iADDU, iSUBU, iADDS, iSUBS, iADDWCS, iSUBWCS,

	EXTOP_ER, EXTHS_ER, EXTHZ_ER, EXTBS_ER, EXTBZ_ER,
	MOVOP_ER, MOV_ER, MOVIBSE_ER, MOVIBZE_ER, MOVIBLS_ER, SWAPB_ER, SWAPH_ER,
	LOGOP_ER, XOR_ER, OR_ER, AND_ER, TEST_ER, SET_ER, CLR_ER, NOTU_ER,
	SHOP_ER, SHLA_ER, SHRA_ER, SHRL_ER, ROR_ER, ROL_ER,
	ADDOP_ER, ADDU_ER, SUBU_ER, ADDS_ER, SUBS_ER, ADDWCS_ER, SUBWCS_ER, NEGS_ER,

	CLK,
	RSTN
);

	output		EXCEPT_BJDS_WR;
	input		iEXCEPT_M;
	output		RFE_MR;
        output		II_EXP_MR, SYS_MR, BRK_MR;
        output [2:0]	EXT_INT_MR;

	input		EXT_PIPE_STALL;
	output		iLD_DELAY_SLOT_STALL;
	output		iMFSR_DELAY_SLOT_STALL;
	output		iLSU_SIGN_EXT_D;
	
	input  [2:0]	ASYNC_EXT_INTN;
	input		INT_ENABLE_MR;

	output		iSPRG_RD_D, iSPRG_WR_D;
	output		SPRG_RD_ER, SPRG_RD_MR, SPRG_WR_ER;
	output [5:0]	PAIR_CNST_ER;
	output		LDST_PAIR_REQ_ER;

	output		JAL_JALR_MR, iJAL_JALR_D;
	output		JAL_ER;
	output		BF_MR, BNF_MR;
	output		BF_ER, BNF_ER;
        output		iBF_D, iBNF_D, iJMP_D, iJAL_D, iJALR_JR_D, iBF_BNF_D;

	input  [31:0]	INU_DATA;
	input  [31:0]	IR_DR;
	output [31:0]	IR_ER;
	input		STALL_DEC;

	output [31:0]	iIMME_CNST_D;
	output		iIALU_IMME_CONST;
	output		iSTOR_IMME_CONST;
	output [3:0]	iRF_AADR, iRF_BADR;
	output		iRF_AREN, iRF_BREN;
	output [3:0]	iRF_WADR_M;
	output [3:0]	RF_WADR_ER, RF_WADR_MR, RF_WADR_WR;
	output		iRF_WEN_M;
	output 		RF_WEN_ER, RF_WEN_MR, RF_WEN_WR;

	input		LSU_LOADREQ;
	output		iLOADREQ_D, iSTORREQ_D;
	output [1:0]	iLDST_SZ_D;
	output		STORPAI_ER, STORPAI_MR;

	output		SFU_ER, SFS_ER;
	output		iEXTOP, iEXTHS, iEXTHZ, iEXTBS, iEXTBZ;
	output		iMOVOP, iMOV, iMOVIBSE, iMOVIBZE, iMOVIBLS, iSWAPB, iSWAPH;
	output		iLOGOP, iXOR, iOR, iAND, iTEST, iSET, iCLR;
	output		iSHOP, iSHLA, iSHRA, iSHRL, iROR, iROL;
	output		iADDOP, iADDU, iSUBU, iADDS, iSUBS, iADDWCS, iSUBWCS;

	output		EXTOP_ER, EXTHS_ER, EXTHZ_ER, EXTBS_ER, EXTBZ_ER;
	output		MOVOP_ER, MOV_ER, MOVIBSE_ER, MOVIBLS_ER, MOVIBZE_ER, SWAPB_ER, SWAPH_ER;
	output		LOGOP_ER, XOR_ER, OR_ER, AND_ER, TEST_ER, SET_ER, CLR_ER, NOTU_ER;
	output		SHOP_ER, SHLA_ER, SHRA_ER, SHRL_ER, ROR_ER, ROL_ER;
	output		ADDOP_ER, ADDU_ER, SUBU_ER, ADDS_ER, SUBS_ER, ADDWCS_ER, SUBWCS_ER, NEGS_ER;

	input		CLK, RSTN;

//* internal signals
	reg   		EXTOP_ER, EXTHS_ER, EXTHZ_ER, EXTBS_ER, EXTBZ_ER;
	reg   		MOVOP_ER, MOV_ER;
	reg   		LOGOP_ER, XOR_ER, OR_ER, AND_ER;
	reg   		SHOP_ER, SHLA_ER, SHRA_ER, SHRL_ER, ROR_ER, ROL_ER;
	reg   		ADDOP_ER, ADDU_ER, SUBU_ER, ADDS_ER, SUBS_ER, ADDWCS_ER, SUBWCS_ER;

	reg		SFU_ER, SFS_ER;
	reg    [31:0]	IR_ER, iIR_ER;
	
	reg    [3:0]	iRF_AADR, iRF_BADR, iRF_WADR;
	reg		iRF_AREN, iRF_BREN, iRF_WEN;
	reg    [3:0]	RF_WADR_ER, iRF_WADR_ER;
	reg    [3:0]	RF_WADR_MR, iRF_WADR_MR;
	reg    [3:0]	RF_WADR_WR, iRF_WADR_WR;
	reg		RF_WEN_ER, iRF_WEN_ER;
	reg		RF_WEN_MR, iRF_WEN_MR;
	reg		RF_WEN_WR, iRF_WEN_WR;

	reg		SFU, SFS;
	reg		EXTOP, EXTHS, EXTHZ, EXTBS, EXTBZ;
	reg		MOVOP, MOV;
	reg		LOGOP, XOR, OR, AND;
	reg		SHOP, SHLA, SHRA, SHRL, ROR, ROL;
	reg		ADDOP, ADDU, SUBU, ADDS, SUBS, ADDWCS, SUBWCS;
	
	reg		iSFU, iSFS;
	reg		iEXTOP, iEXTHS, iEXTHZ, iEXTBS, iEXTBZ;
	reg		iMOVOP, iMOV;
	reg		iLOGOP, iXOR, iOR, iAND;
	reg		iSHOP, iSHLA, iSHRA, iSHRL, iROR, iROL;
	reg		iADDOP, iADDU, iSUBU, iADDS, iSUBS, iADDWCS, iSUBWCS;

	reg		SPRG_RD_ER, iSPRG_RD_ER;
	reg		SPRG_RD_MR, iSPRG_RD_MR;
	reg		iLOADREQ_D, iSTORREQ_D;
	reg   [1:0]	iLDST_SZ_D;
	reg   		iIALU_IMME_CONST;
	reg   [31:0]	iIMME_CNST_D;
	reg   		iRF_WEN_M;
	reg   [3:0]	iRF_WADR_M;
	reg		iBF_D, BF_D;
	reg		iBNF_D, BNF_D;
	reg		JAL_ER;
	reg		BF_ER, iBF_ER;
	reg		BNF_ER, iBNF_ER;
	reg		BF_MR, BNF_MR;
	reg		iJAL_JALR_D, JAL_JALR_ER, JAL_JALR_MR;
        reg		iJMP_D, iJAL_D, iJALR_JR_D, iBF_BNF_D;
	reg		iSTOR_IMME_CONST;

	reg		iMOVIBZE, iMOVIBSE, iMOVIBLS, iSWAPH, iSWAPB, iNOTU, iNEGS;
	reg		MOVIBSE_ER, MOVIBZE_ER, MOVIBLS_ER, SWAPH_ER, SWAPB_ER;
	reg		NOTU_ER, NEGS_ER;
	reg		iTEST, iCLR, iSET;
	reg		TEST_ER, CLR_ER, SET_ER;
	reg		LD_DELAY_SLOT_STALL, iLD_DELAY_SLOT_STALL;
	reg		iSPRG_WR_D, iSPRG_RD_D;
	reg		SPRG_WR_ER;
	reg   [5:0]	PAIR_CNST_ER, iPAIR_CNST_D;
	reg		iLDST_PAIR_REQ_D, LDST_PAIR_REQ_ER;
	reg		iSTORPAI_D;
	reg		iSTORPAI_ER, STORPAI_ER;
	reg		iSTORPAI_MR, STORPAI_MR;
	reg		II_EXP_ER, iII_EXP_D;
	reg		II_EXP_MR;
	reg		iBRK_D, BRK_ER, BRK_MR;
	reg		iSYS_D, SYS_ER, SYS_MR;
	reg		iRFE_D, RFE_ER, RFE_MR;
	reg   [2:0]	EXT_INT_ER;
	reg   [2:0]	EXT_INT_MR;
	reg		MFSR_DELAY_SLOT_STALL, iMFSR_DELAY_SLOT_STALL;
	reg		JMP_ER;
	reg		BRJP_MR, BRJP_WR;
	reg		EXCEPT_BJDS_WR, iEXCEPT_BJDS_WR;
	reg		iLSU_SIGN_EXT_D;

	always @(posedge CLK or negedge RSTN)
	if(RSTN == 1'b0) begin
		EXTOP_ER	<= #(`UD) 1'b0;
		EXTHS_ER	<= #(`UD) 1'b0;
		EXTHZ_ER	<= #(`UD) 1'b0;
		EXTBS_ER	<= #(`UD) 1'b0;
		EXTBZ_ER	<= #(`UD) 1'b0;
		
		MOVOP_ER	<= #(`UD) 1'b0;
		MOV_ER		<= #(`UD) 1'b0;
		MOVIBSE_ER	<= #(`UD) 1'b0;
		MOVIBZE_ER	<= #(`UD) 1'b0;
		MOVIBLS_ER	<= #(`UD) 1'b0;

		SWAPB_ER	<= #(`UD) 1'b0;
		SWAPH_ER	<= #(`UD) 1'b0;
		
		LOGOP_ER	<= #(`UD) 1'b0;
		XOR_ER		<= #(`UD) 1'b0;
		OR_ER		<= #(`UD) 1'b0;
		AND_ER		<= #(`UD) 1'b0;
		TEST_ER		<= #(`UD) 1'b0;
		SET_ER		<= #(`UD) 1'b0;
		CLR_ER		<= #(`UD) 1'b0;
		NOTU_ER		<= #(`UD) 1'b0;
		NEGS_ER		<= #(`UD) 1'b0;

		SHOP_ER		<= #(`UD) 1'b0;
		SHLA_ER		<= #(`UD) 1'b0;
		SHRA_ER		<= #(`UD) 1'b0;
		SHRL_ER		<= #(`UD) 1'b0;
		ROR_ER		<= #(`UD) 1'b0;
		ROL_ER		<= #(`UD) 1'b0;
		ADDOP_ER	<= #(`UD) 1'b0;
		ADDU_ER		<= #(`UD) 1'b0;
		SUBU_ER		<= #(`UD) 1'b0;
		ADDS_ER		<= #(`UD) 1'b0;
		SUBS_ER		<= #(`UD) 1'b0;
		ADDWCS_ER	<= #(`UD) 1'b0;
		SUBWCS_ER	<= #(`UD) 1'b0;

		SPRG_WR_ER	<= #(`UD) 1'b0;
		LDST_PAIR_REQ_ER<= #(`UD) 1'b0;
		STORPAI_ER	<= #(`UD) 1'b0;
		STORPAI_MR	<= #(`UD) 1'b0;
	end
	else if(~STALL_DEC) begin
		EXTOP_ER	<= #(`UD) iEXTOP;
		EXTHS_ER	<= #(`UD) iEXTHS;
		EXTHZ_ER	<= #(`UD) iEXTHZ;
		EXTBS_ER	<= #(`UD) iEXTBS;
		EXTBZ_ER	<= #(`UD) iEXTBZ;
		
		MOVOP_ER	<= #(`UD) iMOVOP;
		MOV_ER		<= #(`UD) iMOV;
		MOVIBSE_ER	<= #(`UD) iMOVIBSE;
		MOVIBZE_ER	<= #(`UD) iMOVIBZE;
		MOVIBLS_ER	<= #(`UD) iMOVIBLS;
		
		SWAPB_ER	<= #(`UD) iSWAPB;
		SWAPH_ER	<= #(`UD) iSWAPH;

		LOGOP_ER	<= #(`UD) iLOGOP;
		XOR_ER		<= #(`UD) iXOR;
		OR_ER		<= #(`UD) iOR;
		AND_ER		<= #(`UD) iAND;
		TEST_ER		<= #(`UD) iTEST;
		SET_ER		<= #(`UD) iSET;
		CLR_ER		<= #(`UD) iCLR;
		NOTU_ER		<= #(`UD) iNOTU;
		NEGS_ER		<= #(`UD) iNEGS;

		SHOP_ER		<= #(`UD) iSHOP;
		SHLA_ER		<= #(`UD) iSHLA;
		SHRA_ER		<= #(`UD) iSHRA;
		SHRL_ER		<= #(`UD) iSHRL;
		ROR_ER		<= #(`UD) iROR;
		ROL_ER		<= #(`UD) iROL;
		ADDOP_ER	<= #(`UD) iADDOP;
		ADDU_ER		<= #(`UD) iADDU;
		SUBU_ER		<= #(`UD) iSUBU;
		ADDS_ER		<= #(`UD) iADDS;
		SUBS_ER		<= #(`UD) iSUBS;
		ADDWCS_ER	<= #(`UD) iADDWCS;
		SUBWCS_ER	<= #(`UD) iSUBWCS;

		SPRG_WR_ER	<= #(`UD) iSPRG_WR_D;

		PAIR_CNST_ER	<= #(`UD) iPAIR_CNST_D;
		LDST_PAIR_REQ_ER<= #(`UD) iLDST_PAIR_REQ_D;
		STORPAI_ER	<= #(`UD) iSTORPAI_D;

		STORPAI_MR	<= #(`UD) ~iEXCEPT_M & STORPAI_ER;
	end

	always @(posedge CLK or negedge RSTN)
	if(RSTN == 1'b0) begin
		JAL_ER		<= #(`UD) 1'b0;

		BF_ER		<= #(`UD) 1'b0;
		BF_MR		<= #(`UD) 1'b0;
		BNF_ER		<= #(`UD) 1'b0;
		BNF_MR		<= #(`UD) 1'b0;

		JMP_ER     	<= #(`UD) 1'b0;
		JAL_JALR_ER	<= #(`UD) 1'b0;
		JAL_JALR_MR	<= #(`UD) 1'b0;

		II_EXP_ER	<= #(`UD) 1'b0;
		II_EXP_MR	<= #(`UD) 1'b0;
		BRK_ER   	<= #(`UD) 1'b0;
		BRK_MR   	<= #(`UD) 1'b0;
		SYS_ER   	<= #(`UD) 1'b0;
		SYS_MR   	<= #(`UD) 1'b0;
		RFE_ER   	<= #(`UD) 1'b0;
		RFE_MR   	<= #(`UD) 1'b0;
		EXT_INT_ER	<= #(`UD) 3'b000;
		EXT_INT_MR	<= #(`UD) 3'b000;
		BRJP_MR		<= #(`UD) 1'b0;
		BRJP_WR		<= #(`UD) 1'b0;
		EXCEPT_BJDS_WR	<= #(`UD) 1'b0;
	end
	else if(~STALL_DEC) begin
		BRJP_MR		<= #(`UD) (BF_ER | BNF_ER | JAL_JALR_ER | JMP_ER);
		BRJP_WR		<= #(`UD) BRJP_MR;
		EXCEPT_BJDS_WR	<= #(`UD) iEXCEPT_BJDS_WR;

		JAL_ER		<= #(`UD) iJAL_D;
		BF_ER		<= #(`UD) iBF_ER;
		BF_MR		<= #(`UD) BF_ER;
		BNF_ER		<= #(`UD) iBNF_ER;
		BNF_MR		<= #(`UD) BNF_ER;

		JMP_ER     	<= #(`UD) iJMP_D;
		JAL_JALR_ER	<= #(`UD) iJAL_JALR_D;
		JAL_JALR_MR	<= #(`UD) JAL_JALR_ER;

		II_EXP_ER	<= #(`UD) iII_EXP_D;
		II_EXP_MR	<= #(`UD) ~iEXCEPT_M & II_EXP_ER;
		BRK_ER   	<= #(`UD) iBRK_D;
		BRK_MR   	<= #(`UD) ~iEXCEPT_M & BRK_ER;
		SYS_ER   	<= #(`UD) iSYS_D;
		SYS_MR   	<= #(`UD) ~iEXCEPT_M & SYS_ER;
		RFE_ER   	<= #(`UD) iRFE_D;
		RFE_MR   	<= #(`UD) RFE_ER;
		
		EXT_INT_ER	<= #(`UD) ~ASYNC_EXT_INTN; // invert
		EXT_INT_MR	<= #(`UD) (~INT_ENABLE_MR || INT_ENABLE_MR & iEXCEPT_M) 
					   ? 3'b000 : EXT_INT_ER;
	end

	always @(posedge CLK or negedge RSTN)
	if(RSTN == 1'b0) begin
		IR_ER		<= #(`UD) {`H_NOP, `H_NOP};

		RF_WADR_ER	<= #(`UD) 4'b0;
		RF_WADR_MR	<= #(`UD) 4'b0;
		RF_WADR_WR	<= #(`UD) 4'b0;
		RF_WEN_ER	<= #(`UD) 1'b1;
		RF_WEN_MR	<= #(`UD) 1'b1;
		RF_WEN_WR	<= #(`UD) 1'b1;

		SFU_ER		<= #(`UD) 1'b0;
		SFS_ER		<= #(`UD) 1'b0;

	end
	//else if(~STALL_DEC) begin
	else begin

		SFU_ER		<= #(`UD) (STALL_DEC) ? SFU_ER : iSFU;
		SFS_ER		<= #(`UD) (STALL_DEC) ? SFS_ER : iSFS;

		IR_ER		<= #(`UD) (STALL_DEC) ? IR_ER : iIR_ER;

		RF_WADR_ER	<= #(`UD) (STALL_DEC) ? RF_WADR_ER : iRF_WADR_ER;
		RF_WADR_MR	<= #(`UD) (STALL_DEC) ? RF_WADR_MR : iRF_WADR_MR;
		RF_WEN_ER	<= #(`UD) (STALL_DEC) ? RF_WEN_ER : iRF_WEN_ER;

	// the following modification is for (CASE 1) deadlock,
	// write-back without STALL

		RF_WEN_MR	<= #(`UD) (STALL_DEC) ? 
				//(iMFSR_DELAY_SLOT_STALL | RF_WEN_MR) : iRF_WEN_MR;
				(iMFSR_DELAY_SLOT_STALL | iLD_DELAY_SLOT_STALL | RF_WEN_MR) : iRF_WEN_MR;


	// if you modify here, remember to modify DPFOR.v's RF_WDI_WR
	//
		//RF_WADR_WR	<= #(`UD) (STALL_DEC) ? RF_WADR_WR : iRF_WADR_WR;
		//RF_WADR_WR	<= #(`UD) (STALL_DEC & ~iMFSR_DELAY_SLOT_STALL) ? RF_WADR_WR : iRF_WADR_WR; 
		RF_WADR_WR	<= #(`UD) (STALL_DEC & ~iMFSR_DELAY_SLOT_STALL & ~iLD_DELAY_SLOT_STALL) ? RF_WADR_WR : iRF_WADR_WR; 

		//RF_WEN_WR	<= #(`UD) (STALL_DEC) ? RF_WEN_WR : iRF_WEN_WR;
		//RF_WEN_WR	<= #(`UD) (STALL_DEC & ~iMFSR_DELAY_SLOT_STALL) ? RF_WEN_WR : iRF_WEN_WR;
		RF_WEN_WR	<= #(`UD) (STALL_DEC & ~iMFSR_DELAY_SLOT_STALL & ~iLD_DELAY_SLOT_STALL) ? RF_WEN_WR : iRF_WEN_WR;
	end

	always @(posedge CLK or negedge RSTN)
	if(RSTN == 1'b0) begin
		SPRG_RD_ER	<= #(`UD) 1'b0;
		SPRG_RD_MR	<= #(`UD) 1'b0;
	end
	else begin //if(~EXT_PIPE_STALL) begin
		//SPRG_RD_ER	<= #(`UD) (EXT_PIPE_STALL) ? SPRG_RD_ER : iSPRG_RD_D;
		//SPRG_RD_MR	<= #(`UD) (EXT_PIPE_STALL) ? SPRG_RD_MR : iSPRG_RD_MR;
		SPRG_RD_ER	<= #(`UD) (STALL_DEC) ? SPRG_RD_ER : iSPRG_RD_D;
		SPRG_RD_MR	<= #(`UD) (STALL_DEC) ? SPRG_RD_MR : iSPRG_RD_MR;
		//SPRG_RD_MR	<= #(`UD) (EXT_PIPE_STALL) ? SPRG_RD_MR : iSPRG_RD_MR;
	end

	always @(posedge CLK or negedge RSTN)
	if(RSTN == 1'b0) begin
		LD_DELAY_SLOT_STALL 	<= #(`UD) 1'b0;
		MFSR_DELAY_SLOT_STALL	<= #(`UD) 1'b0;
	end
	else if(~EXT_PIPE_STALL) begin
		LD_DELAY_SLOT_STALL 	<= #(`UD) iLD_DELAY_SLOT_STALL;
		MFSR_DELAY_SLOT_STALL	<= #(`UD) iMFSR_DELAY_SLOT_STALL;
	end

always @(LSU_LOADREQ or iRF_AADR or RF_WADR_ER or iRF_AREN or iRF_BREN or
	 LD_DELAY_SLOT_STALL or
	 MFSR_DELAY_SLOT_STALL or SPRG_RD_ER or iRF_BADR
) begin
// use hardware detect load delay slot(increase critical path)
	if(~LD_DELAY_SLOT_STALL &&
	    LSU_LOADREQ && ((iRF_AADR == RF_WADR_ER) && iRF_AREN ||
	 		    (iRF_BADR == RF_WADR_ER) && iRF_BREN)
	  )
	  	iLD_DELAY_SLOT_STALL = 1'b1;
	else
	  	iLD_DELAY_SLOT_STALL = 1'b0;

	//iSPRG_RD_D = iMFSR_D

	if(~MFSR_DELAY_SLOT_STALL &&
	   SPRG_RD_ER && ((iRF_AADR == RF_WADR_ER) && iRF_AREN ||
                          (iRF_BADR == RF_WADR_ER) && iRF_BREN)
	  )
		iMFSR_DELAY_SLOT_STALL = 1'b1;
	else
		iMFSR_DELAY_SLOT_STALL = 1'b0;
end

always @(STALL_DEC or IR_DR or INU_DATA or IR_ER or
	 iSPRG_RD_D or SPRG_RD_ER or SPRG_RD_MR or
	 iBF_D or iBNF_D or iEXCEPT_M or BRJP_WR or EXCEPT_BJDS_WR
) begin

	iIR_ER = IR_DR;

	iSPRG_RD_ER = iSPRG_RD_D;
	iSPRG_RD_MR = SPRG_RD_ER;

	iBF_ER 	= iBF_D;
	iBNF_ER = iBNF_D;

	if(iEXCEPT_M)
		iEXCEPT_BJDS_WR = BRJP_WR;
	else
		iEXCEPT_BJDS_WR = EXCEPT_BJDS_WR;
	
end

always @(STALL_DEC or iRF_WADR or iRF_WEN or
	 RF_WADR_ER or RF_WADR_MR or RF_WADR_WR or
	 RF_WEN_ER or RF_WEN_MR or RF_WEN_WR or iEXCEPT_M
) begin
	iRF_WADR_ER = iRF_WADR;

	iRF_WADR_MR = RF_WADR_ER;

	iRF_WADR_WR = RF_WADR_MR;
	
	iRF_WEN_ER  = iEXCEPT_M | iRF_WEN;	

	iRF_WEN_MR  = iEXCEPT_M | RF_WEN_ER;	

	iRF_WEN_WR  = iEXCEPT_M | RF_WEN_MR;	

	iRF_WADR_M = iRF_WADR_WR;
	iRF_WEN_M  = RF_WEN_MR;
end

// `define H_NOP 16'hCA04
wire [3:0] OP          = (iEXCEPT_M) ? 4'hC : IR_DR[31:28];
wire [3:0] SUBOP       = (iEXCEPT_M) ? 4'hA : IR_DR[27:24];
wire [3:0] SUBA	       = (iEXCEPT_M) ? 4'h0 : IR_DR[23:20];
wire [3:0] SUBSUBOP    = (iEXCEPT_M) ? 4'h4 : IR_DR[19:16];

//**********************
//* insn decoding
//**********************

always @(IR_DR or OP or SUBOP or SUBA or SUBSUBOP or iIALU_IMME_CONST or iSHOP) begin
	if(OP == `H_OP_SHIFT_ALUI && (SUBOP == `H_SUBOP_XORI || SUBOP == `H_SUBOP_ORI || SUBOP == `H_SUBOP_ANDI))
		iRF_AADR = 4'd15;
	else if(OP == `H_OP_LOAD 
		//|| OP == `H_OP_SHIFT_ALUI && (SUBOP == `H_SUBOP_NOTU || SUBOP == `H_SUBOP_NEGS)
	       )
	        //(SUBOP == `H_SUBOP_LW || SUBOP == `H_SUBOP_LHZ || SUBOP == `H_SUBOP_LBZ))
		iRF_AADR = IR_DR[19:16];
	else
		iRF_AADR = IR_DR[23:20];

	iRF_BADR = IR_DR[19:16];

	if(OP == `H_OP_SHIFT_ALUI && (SUBOP == `H_SUBOP_XORI || SUBOP == `H_SUBOP_ORI || SUBOP == `H_SUBOP_ANDI))
		iRF_WADR = 4'd15; // for xori/ori/andi
	else if(OP == `H_OP_JAL ||
		OP == `H_OP_SF_EXT && SUBOP == `H_SUBOP_EXT_JALR_JR && SUBSUBOP == `H_SUBSUBOP_JALR)
		iRF_WADR = 4'd8; // for link register
	else
		iRF_WADR = IR_DR[23:20];

	if(OP == `H_OP_LOAD) begin
	  if(SUBA == `H_SUBA_LBZ)
		iIMME_CNST_D = {{28{IR_DR[27]}}, IR_DR[27:24]};
	  else if(SUBA == `H_SUBA_LHZ)
		iIMME_CNST_D = {{27{IR_DR[27]}}, IR_DR[27:24], 1'b0};
	  else
		iIMME_CNST_D = {{26{IR_DR[27]}}, IR_DR[27:24], 2'b00};
	end
	else if(OP == `H_OP_STOR) begin
	  if(SUBSUBOP == `H_SUBSUBOP_SB)
		iIMME_CNST_D = {{28{IR_DR[27]}}, IR_DR[27:24]};
	  else if(SUBSUBOP == `H_SUBSUBOP_SH)
		iIMME_CNST_D = {{27{IR_DR[27]}}, IR_DR[27:24], 1'b0};
	  else
		iIMME_CNST_D = {{26{IR_DR[27]}}, IR_DR[27:24], 2'b00};
	end
	else if(OP == `H_OP_ADDIS || OP == `H_OP_MOVIBSE)
		iIMME_CNST_D = {{24{IR_DR[27]}}, IR_DR[27:24], IR_DR[19:16]};
	else if(OP == `H_OP_MOVIBZE ||
	        OP == `H_OP_MOVIBLS ||
	        OP == `H_OP_ADDIU    ||

	       (OP == `H_OP_SHIFT_ALUI && 
				 (SUBOP[3:1] == `H_SUBOP_SHLAI3 ||
		 		  SUBOP[3:1] == `H_SUBOP_SHRAI3 ||
		 		  SUBOP[3:1] == `H_SUBOP_SHRLI3)) 
				||
	       (OP == `H_OP_ADDSUB_TEST && 
				 (SUBOP[3:1] == `H_SUBOP_TESTI3 ||
				  SUBOP[3:1] == `H_SUBOP_CLRI3  ||
				  SUBOP[3:1] == `H_SUBOP_SETI3))
	       )
		iIMME_CNST_D = {24'h0, IR_DR[27:24], IR_DR[19:16]};

	else if(OP == `H_OP_LOAD_STOR_PAI && 
		(SUBOP == `H_SUBOP_SWPAI || SUBOP == `H_SUBOP_SHPAI || SUBOP == `H_SUBOP_SBPAI))
		iIMME_CNST_D = {29'b0, (SUBOP == `H_SUBOP_SWPAI) 
					 ? 3'b100 : 
					(SUBOP == `H_SUBOP_SHPAI) ? 3'b010 : 3'b001};
	else
		iIMME_CNST_D = {24'h0, IR_DR[23:16]};

	// u could optimize here
	if(OP == `H_OP_LOAD_STOR_PAI)
		case(SUBOP)
		`H_SUBOP_LWPAIN:  iPAIR_CNST_D = {IR_DR[19:16], 2'b0};
		`H_SUBOP_LHZPAIN,
		`H_SUBOP_LHSPAIN: iPAIR_CNST_D = {IR_DR[19], IR_DR[19:16], 1'b0};
		`H_SUBOP_LBZPAIN,
		`H_SUBOP_LBSPAIN: iPAIR_CNST_D = {{2{IR_DR[19]}}, IR_DR[19:16]};
		`H_SUBOP_SWPAIN:  iPAIR_CNST_D = {IR_DR[23:20], 2'b0};
		`H_SUBOP_SHPAIN:  iPAIR_CNST_D = {IR_DR[23], IR_DR[23:20], 1'b0};
		`H_SUBOP_SBPAIN:  iPAIR_CNST_D = {{2{IR_DR[23]}}, IR_DR[23:20]};
		default:	  iPAIR_CNST_D = 6'hxx;
		endcase
	else
		iPAIR_CNST_D = 6'hxx;
end


always @(OP or SUBOP or SUBSUBOP or SUBA or iEXCEPT_M
) begin
	iSFU 		= 1'b0;
	iSFS 		= 1'b0;
	iEXTOP  	= 1'b0;
	iEXTHS		= 1'b0;
	iEXTHZ		= 1'b0;
	iEXTBS 		= 1'b0;
	iEXTBZ 		= 1'b0;

	iMOVOP		= 1'b0;
	iMOV    	= 1'b0;
	iMOVIBZE	= 1'b0;
	iMOVIBSE	= 1'b0;
	iMOVIBLS	= 1'b0;
	iSWAPH		= 1'b0;
	iSWAPB		= 1'b0;

	iLOGOP		= 1'b0;
	iXOR		= 1'b0;
	iOR		= 1'b0;
	iAND		= 1'b0;
	iNOTU		= 1'b0;
	iCLR		= 1'b0;
	iSET		= 1'b0;
	iTEST		= 1'b0;

	iSHOP		= 1'b0;
	iSHLA		= 1'b0;
	iSHRA		= 1'b0;
	iSHRL		= 1'b0;
	iROR 		= 1'b0;
	iROL 		= 1'b0;

	iADDOP		= 1'b0;
	iADDU		= 1'b0;
	iSUBU		= 1'b0;
	iADDS		= 1'b0;
	iSUBS		= 1'b0;
	iADDWCS		= 1'b0;
	iSUBWCS		= 1'b0;
	iNEGS		= 1'b0;

	iSPRG_RD_D		= 1'b0;
	iBF_D		= 1'b0;
	iBNF_D		= 1'b0;

        iJMP_D		= 1'b0;
        iJAL_D		= 1'b0;
        iJALR_JR_D	= 1'b0;
        iJAL_JALR_D	= 1'b0;
        iBF_BNF_D	= 1'b0;

	iSPRG_WR_D	= 1'b0;
	iSPRG_RD_D	= 1'b0;
	iBRK_D		= 1'b0;
	iSYS_D		= 1'b0;
	iRFE_D		= 1'b0;
	
	
	case(OP) // synopsys parallel_case
	`H_OP_LOAD:   begin
			iADDOP = 1'b1; iADDS = 1'b1;
			iRF_AREN = 1'b1; iRF_BREN = 1'b0; iRF_WEN = 1'b0; 
		      end
	`H_OP_STOR:   begin
			iADDOP = 1'b1; iADDS = 1'b1;
			iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b1; 
		      end
	`H_OP_MOVIBZE:
                      begin
                        iMOVOP = 1'b1; iMOVIBZE = 1'b1;
                        iRF_AREN = 1'b1; iRF_BREN = 1'b0; iRF_WEN = 1'b0;
                      end
	`H_OP_MOVIBLS:
		      begin
			iMOVOP = 1'b1; iMOVIBLS = 1'b1;
			iRF_AREN = 1'b1; iRF_BREN = 1'b0; iRF_WEN = 1'b0;
		      end
	`H_OP_MOVIBSE:begin
			iMOVOP = 1'b1; iMOVIBSE = 1'b1;
			iRF_AREN = 1'b0; iRF_BREN = 1'b0; iRF_WEN = 1'b0;
		      end	
	`H_OP_LOAD_STOR_PAI:
		      begin
		 	case(SUBOP) // synopsys parallel_case
			`H_SUBOP_LWPAIN: begin iRF_AREN = 1'b0; iRF_BREN = 1'b0; iRF_WEN = 1'b0; end
			`H_SUBOP_LHZPAIN:begin iRF_AREN = 1'b0; iRF_BREN = 1'b0; iRF_WEN = 1'b0; end
			`H_SUBOP_LBZPAIN:begin iRF_AREN = 1'b0; iRF_BREN = 1'b0; iRF_WEN = 1'b0; end
			`H_SUBOP_LHSPAIN:begin iRF_AREN = 1'b0; iRF_BREN = 1'b0; iRF_WEN = 1'b0; end
			`H_SUBOP_LBSPAIN:begin iRF_AREN = 1'b0; iRF_BREN = 1'b0; iRF_WEN = 1'b0; end
			`H_SUBOP_SWPAI:  begin iADDOP = 1'b1; iADDS = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
			`H_SUBOP_SHPAI:  begin iADDOP = 1'b1; iADDS = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
			`H_SUBOP_SBPAI:  begin iADDOP = 1'b1; iADDS = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
			`H_SUBOP_SWPAIN: begin iRF_AREN = 1'b0; iRF_BREN = 1'b1; iRF_WEN = 1'b1; end
			`H_SUBOP_SHPAIN: begin iRF_AREN = 1'b0; iRF_BREN = 1'b1; iRF_WEN = 1'b1; end
			`H_SUBOP_SBPAIN: begin iRF_AREN = 1'b0; iRF_BREN = 1'b1; iRF_WEN = 1'b1; end
			default:         begin iRF_AREN = 1'b0; iRF_BREN = 1'b0; iRF_WEN = 1'b1; end
			endcase
		      end
	`H_OP_ADDIU:  begin
			begin iADDOP = 1'b1; iADDU = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
		      end
	`H_OP_ADDIS:  begin
			begin iADDOP = 1'b1; iADDS = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
		      end
	`H_OP_JMP:    begin iJMP_D = 1'b1;                   iRF_AREN = 1'b0; iRF_BREN = 1'b0; iRF_WEN = 1'b1; end
	`H_OP_JAL:    begin iJAL_JALR_D = 1'b1;iJAL_D = 1'b1;iRF_AREN = 1'b0; iRF_BREN = 1'b0; iRF_WEN = 1'b0; end
	`H_OP_BNF:    begin iBF_BNF_D = 1'b1; iBNF_D = 1'b1; iRF_AREN = 1'b0; iRF_BREN = 1'b0; iRF_WEN = 1'b1; end
	`H_OP_BF:     begin iBF_BNF_D = 1'b1; iBF_D  = 1'b1; iRF_AREN = 1'b0; iRF_BREN = 1'b0; iRF_WEN = 1'b1; end

	`H_OP_SF_EXT: begin
			case(SUBOP)
			`H_SUBOP_SFEQ,  `H_SUBOP_SFNE,  
			`H_SUBOP_SFGTS, `H_SUBOP_SFGES, 
			`H_SUBOP_SFLTS, `H_SUBOP_SFLES:
				begin
				  iSFS = 1'b1; iADDOP = 1'b1;
				  iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b1; 
			 	end
			`H_SUBOP_SFGTU, `H_SUBOP_SFGEU, 
			`H_SUBOP_SFLTU, `H_SUBOP_SFLEU: 
				begin
				  iSFU = 1'b1; iADDOP = 1'b1;
				  iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b1; 
			 	end
			`H_SUBOP_EXT_JALR_JR: 
				begin
				  case(SUBSUBOP)
				  `H_SUBSUBOP_EXTHS: begin iEXTOP = 1'b1; iEXTHS = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b0; iRF_WEN = 1'b0; end
				  `H_SUBSUBOP_EXTHZ: begin iEXTOP = 1'b1; iEXTHZ = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b0; iRF_WEN = 1'b0; end
				  `H_SUBSUBOP_EXTBS: begin iEXTOP = 1'b1; iEXTBS  = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b0; iRF_WEN = 1'b0; end
				  `H_SUBSUBOP_EXTBZ: begin iEXTOP = 1'b1; iEXTBZ  = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b0; iRF_WEN = 1'b0; end
				  `H_SUBSUBOP_JALR:  begin iJAL_JALR_D = 1'b1; iJALR_JR_D = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b0; iRF_WEN = 1'b0; end
				  `H_SUBSUBOP_JR:    begin iJALR_JR_D = 1'b1;                     iRF_AREN = 1'b1; iRF_BREN = 1'b0; iRF_WEN = 1'b1; end
				  default:begin iRF_AREN = 1'b0; iRF_BREN = 1'b0; iRF_WEN = 1'b1; end
				  endcase
				end
			`H_SUBOP_DIVS,
			`H_SUBOP_DIVU,
			`H_SUBOP_MULS,
			`H_SUBOP_MULU:
				begin
				  iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b0; 
				end
			default:begin iRF_AREN = 1'b0; iRF_BREN = 1'b0; iRF_WEN = 1'b1; end
			endcase
		      end	
	`H_OP_MOV_ALU_EXP:
		      begin
			case(SUBOP)
			`H_SUBOP_BRK:    begin iBRK_D = 1'b1; iRF_AREN = 1'b0; iRF_BREN = 1'b0; iRF_WEN = 1'b1; end
			`H_SUBOP_SYS:    begin iSYS_D = 1'b1; iRF_AREN = 1'b0; iRF_BREN = 1'b0; iRF_WEN = 1'b1; end
			`H_SUBOP_RFE:    begin iRFE_D = (SUBSUBOP == 4'h0 && SUBA == 4'h0) ? 1'b1 : 1'b0; iRF_AREN = 1'b0; iRF_BREN = 1'b0; iRF_WEN = 1'b1; end
			`H_SUBOP_MOV:    begin iMOVOP = 1'b1; iMOV = 1'b1; iRF_AREN = 1'b0; iRF_BREN = 1'b1; iRF_WEN = 1'b0; iMOVOP = 1'b1; end
			`H_SUBOP_MSS:    begin iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b1; end
			`H_SUBOP_MAS:    begin iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b1; end
			`H_SUBOP_MODS:   begin iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
			`H_SUBOP_MODU:   begin iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
			`H_SUBOP_SWAPB:  begin iMOVOP = 1'b1; iSWAPB = 1'b1; iRF_AREN = 1'b0; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
			`H_SUBOP_SWAPH:  begin iMOVOP = 1'b1; iSWAPH = 1'b1; iRF_AREN = 1'b0; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
			`H_SUBOP_SHLA:   begin iSHOP  = 1'b1; iSHLA = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
			`H_SUBOP_SHRA:   begin iSHOP  = 1'b1; iSHRA = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
			`H_SUBOP_SHRL:   begin iSHOP  = 1'b1; iSHRL = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
			`H_SUBOP_ROR:    begin iSHOP  = 1'b1; iROR  = 1'b1;  iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
			`H_SUBOP_ROL:    begin iSHOP  = 1'b1; iROL  = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
			default:         begin iRF_AREN = 1'b0; iRF_BREN = 1'b0; iRF_WEN = 1'b1; end
			endcase
		      end	
	`H_OP_SHIFT_ALUI:  
		      begin
			casex(SUBOP) 
			`H_SUBOP_NOTU: begin iLOGOP = 1'b1; iNOTU = 1'b1; iRF_AREN = 1'b0; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
			`H_SUBOP_NEGS: begin iADDOP = 1'b1; iNEGS = 1'b1; iRF_AREN = 1'b0; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
			`H_SUBOP_XOR : begin iLOGOP = 1'b1; iXOR  = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
			`H_SUBOP_OR  : begin iLOGOP = 1'b1; iOR   = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
			`H_SUBOP_AND : begin iLOGOP = 1'b1; iAND  = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
			`H_SUBOP_XORI: begin iLOGOP = 1'b1; iXOR  = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b0; iRF_WEN = 1'b0; end
			`H_SUBOP_ORI:  begin iLOGOP = 1'b1; iOR   = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b0; iRF_WEN = 1'b0; end
			`H_SUBOP_ANDI: begin iLOGOP = 1'b1; iAND  = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b0; iRF_WEN = 1'b0; end
			`H_SUBOP_SHLAI:begin iSHOP  = 1'b1; iSHLA = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b0; iRF_WEN = 1'b0; end
			`H_SUBOP_SHRAI:begin iSHOP  = 1'b1; iSHRA = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b0; iRF_WEN = 1'b0; end
			`H_SUBOP_SHRLI:begin iSHOP  = 1'b1; iSHRL = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b0; iRF_WEN = 1'b0; end

			`H_SUBOP_MTSR: begin iSPRG_WR_D = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b1; end
			`H_SUBOP_MFSR: begin iSPRG_RD_D = 1'b1; iRF_AREN = 1'b0; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
			default:       begin                    iRF_AREN = 1'b0; iRF_BREN = 1'b0; iRF_WEN = 1'b1; end
			endcase
		      end
	`H_OP_ADDSUB_TEST: 
		      begin 
			casex(SUBOP) // synopsys parallel_case
			`H_SUBOP_ADDS:  begin iADDOP = 1'b1; iADDS   = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
			`H_SUBOP_SUBS:  begin iADDOP = 1'b1; iSUBS   = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
			`H_SUBOP_ADDU:  begin iADDOP = 1'b1; iADDU   = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
			`H_SUBOP_SUBU:  begin iADDOP = 1'b1; iSUBU   = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
			`H_SUBOP_ADDWCS:begin iADDOP = 1'b1; iADDWCS = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
			`H_SUBOP_SUBWCS:begin iADDOP = 1'b1; iSUBWCS = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
			`H_SUBOP_TEST:  begin iLOGOP = 1'b1; iTEST = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b1; end
			`H_SUBOP_TESTI: begin iLOGOP = 1'b1; iTEST = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b0; iRF_WEN = 1'b1; end
			`H_SUBOP_CLR:   begin iLOGOP = 1'b1; iCLR  = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
			`H_SUBOP_CLRI:  begin iLOGOP = 1'b1; iCLR  = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b0; iRF_WEN = 1'b0; end
			`H_SUBOP_SET:   begin iLOGOP = 1'b1; iSET  = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b1; iRF_WEN = 1'b0; end
			`H_SUBOP_SETI:  begin iLOGOP = 1'b1; iSET  = 1'b1; iRF_AREN = 1'b1; iRF_BREN = 1'b0; iRF_WEN = 1'b0; end
			default:        begin iRF_AREN = 1'b0; iRF_BREN = 1'b0; iRF_WEN = 1'b1; end
			endcase
		      end
	default:      begin iRF_AREN = 1'b0; iRF_BREN = 1'b0; iRF_WEN = 1'b1; end
	endcase
end

//* insn size, IALU immediate constant generation
always @(OP or SUBOP or SUBSUBOP or SUBA
) begin
	iLOADREQ_D = 1'b0;
	iSTORREQ_D = 1'b0;
	iLDST_SZ_D = 2'b11;
	iSTOR_IMME_CONST = 1'b0;
	iLDST_PAIR_REQ_D = 1'b0;
	iSTORPAI_D = 1'b0;
	iLSU_SIGN_EXT_D = 1'b0;

	case(OP) // synopsys parallel_case
	`H_OP_LOAD: begin
			case(SUBA)
			`H_SUBA_LHZ: begin iLOADREQ_D = 1'b1; iLDST_SZ_D = 2'b01; iIALU_IMME_CONST = 1'b1; end
			`H_SUBA_LBZ: begin iLOADREQ_D = 1'b1; iLDST_SZ_D = 2'b00; iIALU_IMME_CONST = 1'b1; end
			default:     begin iLOADREQ_D = 1'b1; iLDST_SZ_D = 2'b11; iIALU_IMME_CONST = 1'b1; end
			endcase
		    end
	`H_OP_STOR: begin 
			case(SUBSUBOP)
			`H_SUBSUBOP_SH: begin iSTORREQ_D = 1'b1; iLDST_SZ_D = 2'b01; iSTOR_IMME_CONST = 1'b1; iIALU_IMME_CONST = 1'b0; end
			`H_SUBSUBOP_SB: begin iSTORREQ_D = 1'b1; iLDST_SZ_D = 2'b00; iSTOR_IMME_CONST = 1'b1; iIALU_IMME_CONST = 1'b0; end
			default:        begin iSTORREQ_D = 1'b1; iLDST_SZ_D = 2'b11; iSTOR_IMME_CONST = 1'b1; iIALU_IMME_CONST = 1'b0; end
			endcase
		    end
	`H_OP_LOAD_STOR_PAI:
		    begin
			case(SUBOP)
			`H_SUBOP_LWPAIN: begin iLDST_PAIR_REQ_D = 1'b1; iLOADREQ_D = 1'b1; iLDST_SZ_D = 2'b11; iIALU_IMME_CONST = 1'b1; end
			`H_SUBOP_LHZPAIN:begin iLDST_PAIR_REQ_D = 1'b1; iLOADREQ_D = 1'b1; iLDST_SZ_D = 2'b01; iIALU_IMME_CONST = 1'b1; end
			`H_SUBOP_LBZPAIN:begin iLDST_PAIR_REQ_D = 1'b1; iLOADREQ_D = 1'b1; iLDST_SZ_D = 2'b00; iIALU_IMME_CONST = 1'b1; end
			`H_SUBOP_LHSPAIN:begin iLSU_SIGN_EXT_D  = 1'b1; iLDST_PAIR_REQ_D = 1'b1; iLOADREQ_D = 1'b1; iLDST_SZ_D = 2'b01; iIALU_IMME_CONST = 1'b1; end
			`H_SUBOP_LBSPAIN:begin iLSU_SIGN_EXT_D  = 1'b1; iLDST_PAIR_REQ_D = 1'b1; iLOADREQ_D = 1'b1; iLDST_SZ_D = 2'b00; iIALU_IMME_CONST = 1'b1; end
			`H_SUBOP_SWPAI:  begin iSTORREQ_D = 1'b1; iSTORPAI_D = 1'b1; iLDST_SZ_D = 2'b11; iIALU_IMME_CONST = 1'b1; end
			`H_SUBOP_SHPAI:  begin iSTORREQ_D = 1'b1; iSTORPAI_D = 1'b1; iLDST_SZ_D = 2'b01; iIALU_IMME_CONST = 1'b1; end
			`H_SUBOP_SBPAI:  begin iSTORREQ_D = 1'b1; iSTORPAI_D = 1'b1; iLDST_SZ_D = 2'b00; iIALU_IMME_CONST = 1'b1; end
			`H_SUBOP_SWPAIN: begin iLDST_PAIR_REQ_D = 1'b1; iSTORREQ_D = 1'b1; iLDST_SZ_D = 2'b11; iIALU_IMME_CONST = 1'b1; end
			`H_SUBOP_SHPAIN: begin iLDST_PAIR_REQ_D = 1'b1; iSTORREQ_D = 1'b1; iLDST_SZ_D = 2'b01; iIALU_IMME_CONST = 1'b1; end
			`H_SUBOP_SBPAIN: begin iLDST_PAIR_REQ_D = 1'b1; iSTORREQ_D = 1'b1; iLDST_SZ_D = 2'b00; iIALU_IMME_CONST = 1'b1; end
			default: 	 begin iSTORREQ_D = 1'b0; iLOADREQ_D = 1'b0;iLDST_SZ_D = 2'b11; iIALU_IMME_CONST = 1'b0; end
			endcase
		    end
	`H_OP_BNF,
	`H_OP_BF,
	`H_OP_JAL,
	`H_OP_JMP: begin iIALU_IMME_CONST = 1'b1; end

	`H_OP_MOVIBLS,
	`H_OP_MOVIBZE,
	`H_OP_MOVIBSE,
	`H_OP_ADDIS,
	`H_OP_ADDIU: iIALU_IMME_CONST = 1'b1;

	`H_OP_SHIFT_ALUI:
		begin
		  casex(SUBOP)
		  `H_SUBOP_XORI,
		  `H_SUBOP_ORI,
		  `H_SUBOP_ANDI: begin iIALU_IMME_CONST = 1'b1; end
		  `H_SUBOP_SHLAI,
		  `H_SUBOP_SHRAI,
		  `H_SUBOP_SHRLI:begin iIALU_IMME_CONST = 1'b1; end
		  default:       begin iIALU_IMME_CONST = 1'b0; end
		  endcase
		end
	`H_OP_ADDSUB_TEST:
		begin
		  casex(SUBOP)
		  `H_SUBOP_TESTI,
		  `H_SUBOP_CLRI,
		  `H_SUBOP_SETI: iIALU_IMME_CONST = 1'b1;
		  default: iIALU_IMME_CONST = 1'b0;
		  endcase
		end
	default:begin iIALU_IMME_CONST = 1'b0; end
	endcase
end

/////////////////////////////////////////////////
//* reserved insn trap generation
always @(OP or SUBOP or SUBSUBOP or IR_DR
) begin
	case(OP) // synopsys parallel_case
	4'h5: case(SUBOP) // synopsys parallel_case
		4'hB, 4'hC, 
		4'hD, 4'hE, 4'hF: iII_EXP_D = 1'b1;
	      default: iII_EXP_D = 1'b0;
	      endcase
	4'hC: case(SUBOP) // synopsys parallel_case
	      4'hA: case(SUBSUBOP) // synopsys parallel_case
		    4'h7, 4'h8, 4'h9, 4'hA,
		    4'hB, 4'hC, 4'hD, 4'hE, 4'hF: iII_EXP_D = 1'b1;
		    default: iII_EXP_D = 1'b0;
		    endcase
	      4'hF: iII_EXP_D = 1'b1;
	      default: iII_EXP_D = 1'b0;
	      endcase
	4'hD: case(SUBOP) // synopsys parallel_case
		4'h9: iII_EXP_D = (IR_DR[23:16] != 8'h00) ? 1'b1 : 1'b0;
		4'hF: iII_EXP_D = 1'b1;
		default: iII_EXP_D = 1'b0;
	      endcase
	4'hF: case(SUBOP) // synopsys parallel_case
		4'h7:    iII_EXP_D = 1'b1;
		default: iII_EXP_D = 1'b0;
	      endcase
	default: iII_EXP_D = 1'b0;
	endcase
end
endmodule
