/////////////////////////////////////////////////////////////////////////////
// This file is a part of the OpenRISC 1601 Verilog-HDL model
// Copyright (C) 2000 Jimmy C. Chen and OPENCORES.ORG
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// See the file COPYING for the full details of the license.
//
// OPENCORES.ORG is looking for new open source IP cores and developers that
// would like to help in our mission.
////////////////////////////////////////////////////////////////////////////
// IP core: OpenRISC 1601, a 16-bit instruction set/32-bit datapath RISC 
//          processor core of OR1K family, according to OR16 ISA.
// Features:
//  * fast exception processing (see architecture manual for details)
//  * yet optimized for size (without MMU and Caches)
// 
// Documentation: http://www.opencores.org/cores/or1k/or1601/
////////////////////////////////////////////////////////////////////////////
// File: IU.v
// Author: Jimmy Chen-Min Chen, jimmy@ee.nctu.edu.tw, 06/18/2000
// Description: Integer Unit
//  
// Architecture Notes:
// 	 
//  
//  
// TO DO:
//
////////////////////////////////////////////////////////////////////////////
// Revision History
// $Log$
`include "pool/timescale.v"
`include "pool/sys.h"

module IU(
	ASYNC_EXT_INTN,			// asynchronous external interrupt pins, low active

	INU_ADDR, 
	INU_DATA, 
	INU_REQ, 
	iBIU2INU_BUSY,
	iALL_UPDATE_IFA_W,
    
	iLSU_ADDR_MR,
    	LSU_ADDR, 
	LSU_DI, 
	LSU_DO, 
	LSU_SIZE, 

	LSU_LOADREQ, 
	LSU_STORREQ, 
	BIU2LSU_BUSY,
	LSU_STALL,

	RSTN_D2, 
	CLK
);

parameter datawidth = 32;

	output [31:0]	iLSU_ADDR_MR;
	output		iALL_UPDATE_IFA_W;

	input  [2:0]	ASYNC_EXT_INTN;

	output [31:0]	INU_ADDR;
	input  [31:0]	INU_DATA;
	output		INU_REQ;
	//wire  		INU_REQ = 1'b0;
	input		iBIU2INU_BUSY;

	output [31:0]	LSU_ADDR;
	input  [31:0]	LSU_DI;
	output [31:0]	LSU_DO;
	output [1:0]	LSU_SIZE;
	output		LSU_LOADREQ;
	output		LSU_STORREQ;
	input		BIU2LSU_BUSY;
	output		LSU_STALL;

	input 		RSTN_D2;
	input 		CLK;

//* internal wires
	wire [3:0]	iRF_AADR, iRF_BADR, iRF_WADR_M, RF_WADR_WR;
	wire [31:0]	INU_DATA, IR_DR, IR_ER;
	wire [31:0]	iIMME_CNST_D;
	wire [31:0]	PC;
	wire [31:0]	RF_AO, RF_BO;
	wire [1:0]	iLDST_SZ_D;
	wire [31:0]	iSTOR_MR;
	wire [7:0]	iSTORCON;
	wire [31:0]	MACHI, MACLO;
	wire [31:0]	iAOP_ER, AOP_ER, BOP_ER, iALURES;
	wire [3:0]	RF_WADR_ER, RF_WADR_MR;
	wire [7:0]	iAOPCON, iBOPCON;
	wire [3:0]	LSU_BYTEN;
	wire [31:0]	SPR_RES_M;
	wire [31:0]	RF_WDI_WR, iRF_WDI_WR;
	wire [3:0]	iSEL_RF_WDI_WR;
	wire [31:0]	PC_PLUS_4_MR;
	wire		iMISS_PREDICT_E;
	wire		iBXF_STALL_E, iJAL_STALL_E;
	wire		iLD_DELAY_SLOT_STALL;
	wire [31:0]	iLSU_DI2DP;
	wire [31:0]	PAIR_MR;
	wire [31:0]	EPCR_WR;

	reg		RSTN_D3;
	wire [2:0]	EXT_INT_MR;
	wire [31:0]	iSPRG_ADDR_E;
	wire [5:0]	PAIR_CNST_ER;
	wire		iMFSR_DELAY_SLOT_STALL;
	wire [31:0]	EEAR_MR;

	always @(posedge CLK or negedge RSTN_D2)
	if(RSTN_D2 == 1'b0)
		RSTN_D3	<= #(`UD) 1'b0;
	else
		RSTN_D3	<= #(`UD) RSTN_D2;

wire PIPE_STALL  	= iBIU2INU_BUSY | BIU2LSU_BUSY 
			  | LSU_STALL | iLD_DELAY_SLOT_STALL | iMFSR_DELAY_SLOT_STALL;
wire EXT_PIPE_STALL  	= iBIU2INU_BUSY | BIU2LSU_BUSY;

regfile regfile1(
	.AO	(RF_AO), 
	.AADR	(iRF_AADR), 
	.AREN	(iRF_AREN), 
	.BO	(RF_BO), 
	.BADR	(iRF_BADR), 
	.BREN	(iRF_BREN), 
	//.WEN	(iRF_WEN_M),
	//.WADR	(iRF_WADR_M), 
	//.iWDI	(iRF_WDI_WR), 
	.WEN	(RF_WEN_WR),
	.WADR	(RF_WADR_WR), 
	.WDI	(RF_WDI_WR), 
	.CLK	(CLK)
);

wire DIVZERO_MR = 1'b0;

//* PCIFA: insn fetch address & program counter
PCIFA	PCIFA1(
        .STALL_PCIFA	(PIPE_STALL),
	.EXT_PIPE_STALL	(EXT_PIPE_STALL),

	.iALL_UPDATE_IFA_W	(iALL_UPDATE_IFA_W),
	.iBIU2INU_BUSY	(iBIU2INU_BUSY),
	.EPCR_WR	(EPCR_WR),		// exception program counter

	.DADR_ALIGNEXP_MR	(DADR_ALIGNEXP_MR), 	// data address alignment exception(LSU)
	.II_EXP_MR		(II_EXP_MR), 		// illegal insn exception(DEC)
	.EXT_INT_MR		(EXT_INT_MR), 		// external interrupt(DEC)
	.SYS_MR			(SYS_MR), 		// h.sys(DEC)
	.BRK_MR			(BRK_MR), 		// h.brk(DEC)
	.OVERFL_MR		(OVERFL_MR), 		// overflow exception(IALU)
	.DIVZERO_MR		(DIVZERO_MR),		// divided by zero exception(MAC)

	.iJMP_D		(iJMP_D),		//* Jump/Branch in Decoding stage
	.iJAL_D		(iJAL_D),
	.iJALR_JR_D	(iJALR_JR_D),
	.iBF_BNF_D	(iBF_BNF_D),
	.iJAL_JALR_D	(iJAL_JALR_D),

	.INU_REQ	(INU_REQ),

	.AOP_ER		(AOP_ER),		//* JR/JALR next PC
	.iAOP_ER	(iAOP_ER),		//* JR/JALR next PC

	.INU_DATA	(INU_DATA),
	.IR_DR   	(IR_DR),
        .iIFA_W		(INU_ADDR),
        .PC		(PC),

	.PC_PLUS_4_MR	(PC_PLUS_4_MR),

	.iLD_DELAY_SLOT_STALL	(iLD_DELAY_SLOT_STALL),

        .iMISS_PREDICT_D(iMISS_PREDICT_D),
        .iMISS_PREDICT_E(iMISS_PREDICT_E),

	.iEXCEPT_M	(iEXCEPT_M),

        .CLK		(CLK),
        .RSTN_D2	(RSTN_D2),
        .RSTN_D3	(RSTN_D3)
);

// branch processing unit
BPU	BPU1(
	.STALL_BPU	(PIPE_STALL),

	.iBF_D		(iBF_D),
	.iBNF_D		(iBNF_D),

        .BF_MR		(BF_MR),
        .BNF_MR		(BNF_MR),
        .BF_ER		(BF_ER),
        .BNF_ER		(BNF_ER),
 
	.TEST_ER	(TEST_ER), 
	.SFU_ER		(SFU_ER), 
	.SFS_ER		(SFS_ER),
        .iFLAG_E 	(iFLAG_E),
        .FLAG_MR	(FLAG_MR),
        .FLAG_WR	(FLAG_WR),

        .iMISS_PREDICT_D(iMISS_PREDICT_D),
        .iMISS_PREDICT_E(iMISS_PREDICT_E),
 
        .CLK		(CLK),
        .RSTN 		(RSTN_D2)
);

//* special register group 0
SPRG   SPRG0(
	.STALL_SPRG0	(PIPE_STALL),
	.EXT_PIPE_STALL	(EXT_PIPE_STALL),
	.iMFSR_DELAY_SLOT_STALL	(iMFSR_DELAY_SLOT_STALL),
	
	.INT_ENABLE_MR	(INT_ENABLE_MR),
	.EXT_INT_MR	(EXT_INT_MR),
	.EXCEPT_BJDS_WR	(EXCEPT_BJDS_WR),	// exception in branch/jump delay slot

        .iSPRG_RD_D	(iSPRG_RD_D),
        .iSPRG_WR_D	(iSPRG_WR_D),
	.AOP_ER		(AOP_ER),
	.BOP_ER		(BOP_ER),
        .iSPRG_ADDR_E	(iSPRG_ADDR_E),         // output to decoding stage
        .SPRG_RD_ER	(SPRG_RD_ER),           // read enable at E stage
        .SPRG_RD_MR	(MFSR_MR),           	// read enable at E stage
        .SPRG_WR_ER	(SPRG_WR_ER),           // write enable

        .PAIR_MR	(PAIR_MR),             	// PAIR_MR + increment_const
	.EEAR_MR	(EEAR_MR),


	.RFE_MR		(RFE_MR), 
	.TEST_ER	(TEST_ER), 
	.SFU_ER		(SFU_ER), 
	.SFS_ER		(SFS_ER),
        .iFLAG_E	(iFLAG_E),
        .FLAG_MR	(FLAG_MR),
        .FLAG_WR	(FLAG_WR),

	.iEXCEPT_M	(iEXCEPT_M),
        .iOVERFL_E	(iOVERFL_E),
        .OVERFL_MR	(OVERFL_MR),
        .OVERFL_WR	(OVERFL_WR),

	.NEGS_ER	(NEGS_ER),		// the 7 *_ER signals are used to update CARRY_MR
	.ADDU_ER	(ADDU_ER),
	.SUBU_ER	(SUBU_ER),
	.ADDS_ER	(ADDS_ER),
	.SUBS_ER	(SUBS_ER),
	.ADDWCS_ER	(ADDWCS_ER),
	.SUBWCS_ER	(SUBWCS_ER),

        .iCARRY_E	(iCARRY_E),
        .CARRY_MR	(CARRY_MR),
        .CARRY_WR	(CARRY_WR),

        .MACHI		(MACHI),
        .MACLO		(MACLO),
	.EPCR_WR	(EPCR_WR),		// exception program counter

	.SPR_RES_M	(SPR_RES_M),		// SPR result before E stage

        .CLK		(CLK),
        .RSTN		(RSTN_D2)
);

//* load/store unit
LSU	LSU1(
        .STALL_LSU	(PIPE_STALL),
        .EXT_PIPE_STALL	(EXT_PIPE_STALL),
	.iLD_DELAY_SLOT_STALL(iLD_DELAY_SLOT_STALL),

	.iEXCEPT_M	(iEXCEPT_M),

        .iSPRG_ADDR_E	(iSPRG_ADDR_E),         // input from decoding stage
        .BOP_ER		(BOP_ER),           	// data input
        .SPRG_WR_ER	(SPRG_WR_ER),		

	.PAIR_CNST_ER	(PAIR_CNST_ER),		// PAIR auto-increment constant, it may be shifted left 2bits, so total 6 bits(signed integer)

	.PAIR_MR	(PAIR_MR),

        .iLSU_ADDR_MR	(iLSU_ADDR_MR),
        .LSU_ADDR	(LSU_ADDR),
        .LSU_DO		(LSU_DO),
        .LSU_DI		(LSU_DI),
        .LSU_SIZE	(LSU_SIZE),
        .LSU_BYTEN	(LSU_BYTEN),
        .LSU_LOADREQ	(LSU_LOADREQ), 
        .LSU_LOADREQ_MR	(LSU_LOADREQ_MR), 
        .LSU_STORREQ	(LSU_STORREQ),
        .LSU_STALL	(LSU_STALL),

	.iLSU_DI2DP	(iLSU_DI2DP),

	.AOP_ER		(AOP_ER),	// for swpai/shpai/sbpai's address
        .iALURES	(iALURES),
	.iSTOR_MR	(iSTOR_MR),	//* output to LSU

	.LDST_PAIR_REQ_ER(LDST_PAIR_REQ_ER),
        .iLDST_SZ_D	(iLDST_SZ_D),
        .iLOADREQ_D	(iLOADREQ_D),
	.iLSU_SIGN_EXT_D(iLSU_SIGN_EXT_D),
        .iSTORREQ_D	(iSTORREQ_D),

	.STORPAI_ER	(STORPAI_ER),
	.STORPAI_MR	(STORPAI_MR),

	.DADR_ALIGNEXP_MR	(DADR_ALIGNEXP_MR),	// data address alignment exception
	.EEAR_MR	(EEAR_MR),

        .CLK		(CLK), 
        .RSTN		(RSTN_D2)
);

// decoding 
DEC DEC1(
	.STALL_DEC	(PIPE_STALL),
	.EXT_PIPE_STALL (EXT_PIPE_STALL),

	.iEXCEPT_M	(iEXCEPT_M),
	.EXCEPT_BJDS_WR	(EXCEPT_BJDS_WR),	// exception in branch/jump delay slot
	.RFE_MR		(RFE_MR), 

	.ASYNC_EXT_INTN (ASYNC_EXT_INTN),	// asynchronous external interrupt pins, low active
	.INT_ENABLE_MR	(INT_ENABLE_MR),

        .II_EXP_MR      (II_EXP_MR),            // illegal insn exception(DEC)
        .EXT_INT_MR     (EXT_INT_MR),           // external interrupt(DEC)
        .SYS_MR         (SYS_MR),               // h.sys(DEC)
        .BRK_MR         (BRK_MR),               // h.brk(DEC)

        .iSPRG_RD_D	(iSPRG_RD_D),
        .iSPRG_WR_D	(iSPRG_WR_D),
        .SPRG_RD_ER	(SPRG_RD_ER),           // read enable at E stage
	.SPRG_RD_MR	(MFSR_MR),
        .SPRG_WR_ER	(SPRG_WR_ER),           // write enable

	.PAIR_CNST_ER	(PAIR_CNST_ER),		// PAIR auto-increment constant, it may be shifted left 2bits, so total 6 bits(signed integer)
	.LDST_PAIR_REQ_ER(LDST_PAIR_REQ_ER),

	.INU_DATA	(INU_DATA),
	.IR_DR		(IR_DR),
	.IR_ER		(IR_ER),

	.iIMME_CNST_D		(iIMME_CNST_D),
	.iIALU_IMME_CONST	(iIALU_IMME_CONST),
	.iSTOR_IMME_CONST	(iSTOR_IMME_CONST),
	.iLD_DELAY_SLOT_STALL	(iLD_DELAY_SLOT_STALL),
	.iMFSR_DELAY_SLOT_STALL	(iMFSR_DELAY_SLOT_STALL),

//* register file read/write
	.iRF_AADR	(iRF_AADR), 
	.iRF_BADR	(iRF_BADR),
	.iRF_AREN	(iRF_AREN), 
	.iRF_BREN	(iRF_BREN),
	.iRF_WADR_M	(iRF_WADR_M),
	.iRF_WEN_M	(iRF_WEN_M),

	.RF_WADR_ER	(RF_WADR_ER),
	.RF_WADR_MR	(RF_WADR_MR),
	.RF_WADR_WR	(RF_WADR_WR),

	.RF_WEN_ER	(RF_WEN_ER),
	.RF_WEN_MR	(RF_WEN_MR),
	.RF_WEN_WR	(RF_WEN_WR),

//* decoding for branch
	.JAL_ER		(JAL_ER),
	.BF_MR		(BF_MR),
	.BNF_MR		(BNF_MR),
	.BF_ER		(BF_ER),
	.BNF_ER		(BNF_ER),

	.JAL_JALR_MR	(JAL_JALR_MR),
	.iJAL_JALR_D	(iJAL_JALR_D),

	.iBF_D		(iBF_D),
	.iBNF_D		(iBNF_D),
        .iJMP_D         (iJMP_D),               //* Jump/Branch in Decoding stage
        .iJAL_D         (iJAL_D),
        .iJALR_JR_D     (iJALR_JR_D),
        .iBF_BNF_D      (iBF_BNF_D),

//* decoding for LSU
	.LSU_LOADREQ	(LSU_LOADREQ),
	.iLOADREQ_D	(iLOADREQ_D),
	.iSTORREQ_D	(iSTORREQ_D),
	.iLDST_SZ_D	(iLDST_SZ_D),
	.STORPAI_ER	(STORPAI_ER),
	.STORPAI_MR	(STORPAI_MR),
	.iLSU_SIGN_EXT_D(iLSU_SIGN_EXT_D),

//* decoding for ALU
	.SFU_ER		(SFU_ER), 
	.SFS_ER		(SFS_ER),

	.iEXTOP(iEXTOP), .iEXTHS(iEXTHS), .iEXTHZ(iEXTHZ), .iEXTBS(iEXTBS), .iEXTBZ(iEXTBZ),
	.iMOVOP(iMOVOP), .iMOV(iMOV), .iMOVIBSE(iMOVIBSE), .iMOVIBZE(iMOVIBZE), .iMOVIBLS(iMOVIBLS), .iSWAPB(iSWAPB), .iSWAPH(iSWAPH),
	.iLOGOP(iLOGOP), .iXOR(iXOR), .iOR(iOR), .iAND(iAND), .iTEST(iTEST), .iCLR(iCLR), .iSET(iSET),
	.iSHOP(iSHOP), .iSHLA(iSHLA), .iSHRA(iSHRA), .iSHRL(iSHRL), .iROR(iROR), .iROL(iROL),
	.iADDOP(iADDOP), .iADDU(iADDU), .iSUBU(iSUBU), .iADDS(iADDS), .iSUBS(iSUBS), .iADDWCS(iADDWCS), .iSUBWCS(iSUBWCS),

	.EXTOP_ER(EXTOP_ER), .EXTHS_ER(EXTHS_ER), .EXTHZ_ER(EXTHZ_ER), .EXTBS_ER(EXTBS_ER), .EXTBZ_ER(EXTBZ_ER),
	.MOVOP_ER(MOVOP_ER), .MOV_ER(MOV_ER), .MOVIBSE_ER(MOVIBSE_ER), .MOVIBZE_ER(MOVIBZE_ER), .MOVIBLS_ER(MOVIBLS_ER), .SWAPB_ER(SWAPB_ER), .SWAPH_ER(SWAPH_ER),
	.LOGOP_ER(LOGOP_ER), .XOR_ER(XOR_ER), .OR_ER(OR_ER), .AND_ER(AND_ER), .TEST_ER(TEST_ER), .SET_ER(SET_ER), .CLR_ER(CLR_ER), .NOTU_ER(NOTU_ER),
	.SHOP_ER(SHOP_ER), .SHLA_ER(SHLA_ER), .SHRA_ER(SHRA_ER), .SHRL_ER(SHRL_ER), .ROR_ER(ROR_ER), .ROL_ER(ROL_ER),
	.ADDOP_ER(ADDOP_ER), .ADDU_ER(ADDU_ER), .SUBU_ER(SUBU_ER), .ADDS_ER(ADDS_ER), .SUBS_ER(SUBS_ER), .ADDWCS_ER(ADDWCS_ER), .SUBWCS_ER(SUBWCS_ER), .NEGS_ER(NEGS_ER),

	.CLK		(CLK),
	.RSTN   	(RSTN_D3)
);

//* datapah forward control
DPCON DPCON1(
	.STALL_DPCON		(PIPE_STALL),

        .iAOPCON		(iAOPCON),
        .iBOPCON		(iBOPCON),
	.iSTORCON		(iSTORCON),

        .iIALU_IMME_CONST	(iIALU_IMME_CONST),
	.iSTOR_IMME_CONST	(iSTOR_IMME_CONST),

        .iRF_AADR		(iRF_AADR),
        .iRF_BADR		(iRF_BADR),
        .iRF_AREN		(iRF_AREN),
        .iRF_BREN		(iRF_BREN),
        .iRF_WADR_M		(iRF_WADR_M),
        .iRF_WEN_M		(iRF_WEN_M),

        .RF_WADR_ER		(RF_WADR_ER),
        .RF_WADR_MR		(RF_WADR_MR),
        .RF_WADR_WR		(RF_WADR_WR),
        .RF_WEN_ER		(RF_WEN_ER),
        .RF_WEN_MR		(RF_WEN_MR),
        .RF_WEN_WR		(RF_WEN_WR),

        .iSEL_RF_WDI_WR		(iSEL_RF_WDI_WR),

	.JAL_JALR_MR		(JAL_JALR_MR),
	.LSU_LOADREQ_MR		(LSU_LOADREQ_MR),
	.LSU_LOADREQ		(LSU_LOADREQ),
	.STORPAI_MR		(STORPAI_MR),		// for h.swpai/h.shpai/h.sbpai
	.MFSR_ER		(SPRG_RD_ER),		// MFSR=SPRG_RD_ER
	.MFSR_MR		(MFSR_MR),

        .CLK			(CLK),
        .RSTN			(RSTN_D2)
);

//* datapath forwarding 
DPFOR DPFOR1(
	.STALL_DPFOR	(PIPE_STALL),
	.iMFSR_DELAY_SLOT_STALL	(iMFSR_DELAY_SLOT_STALL),
	.iLD_DELAY_SLOT_STALL	(iLD_DELAY_SLOT_STALL),

	.PC_PLUS_4_MR	(PC_PLUS_4_MR),

        .iAOP_ER	(iAOP_ER),     	//* output to ALU
        .AOP_ER		(AOP_ER),     	//* output to ALU
        .BOP_ER		(BOP_ER),
	.iSTOR_MR	(iSTOR_MR),	//* output to LSU
	.STORPAI_MR	(STORPAI_MR),

        .iAOPCON	(iAOPCON),
        .iBOPCON	(iBOPCON),
	.iSTORCON	(iSTORCON),

        .RF_AO		(RF_AO),     	//* normal from D stage
        .RF_BO		(RF_BO),

        .iIMME_CNST_D	(iIMME_CNST_D), //* normal: const from D stage for B operand

        .iALURES	(iALURES),     //* forward from E stage
        .SPR_RES_M	(SPR_RES_M),    //* forward from E stage

        .iLSU_DI2DP	(iLSU_DI2DP),  	//* forward from M stage
        //.iLSU_DI2DP	(LSU_DI),  	//* forward from M stage
        .iRF_WDI_WR	(iRF_WDI_WR),   //* forward from M stage

        .RF_WDI_WR	(RF_WDI_WR),   	//* forward from W stage

        .iSEL_RF_WDI_WR	(iSEL_RF_WDI_WR),

	.INU_DATA	(INU_DATA),

        .CLK		(CLK),
        .RSTN		(RSTN_D2)
);

//* Integer ALU
IALU IALU1(
	.STALL_IALU	(PIPE_STALL),

//* operation code: the following signals should be the output of D-FF at E-stage
	.EXTOP_ER(EXTOP_ER), .EXTHS_ER(EXTHS_ER), .EXTHZ_ER(EXTHZ_ER), .EXTBS_ER(EXTBS_ER), .EXTBZ_ER(EXTBZ_ER),
	.MOVOP_ER(MOVOP_ER), .MOV_ER(MOV_ER), .MOVIBSE_ER(MOVIBSE_ER), .MOVIBZE_ER(MOVIBZE_ER), .MOVIBLS_ER(MOVIBLS_ER), .SWAPB_ER(SWAPB_ER), .SWAPH_ER(SWAPH_ER),
	.LOGOP_ER(LOGOP_ER), .XOR_ER(XOR_ER), .OR_ER(OR_ER), .AND_ER(AND_ER), .TEST_ER(TEST_ER), .SET_ER(SET_ER), .CLR_ER(CLR_ER), .NOTU_ER(NOTU_ER),
	.SHOP_ER(SHOP_ER), .SHLA_ER(SHLA_ER), .SHRA_ER(SHRA_ER), .SHRL_ER(SHRL_ER), .ROR_ER(ROR_ER), .ROL_ER(ROL_ER),
	.ADDOP_ER(ADDOP_ER), .ADDU_ER(ADDU_ER), .SUBU_ER(SUBU_ER), .ADDS_ER(ADDS_ER), .SUBS_ER(SUBS_ER), .ADDWCS_ER(ADDWCS_ER), .SUBWCS_ER(SUBWCS_ER), .NEGS_ER(NEGS_ER),

//* operands
	.CARRY_MR	(CARRY_MR),
	.CARRY_WR	(CARRY_WR),
	.AOP		(AOP_ER),
	.BOP		(BOP_ER),
	.iALURES	(iALURES),

	.SFU_ER		(SFU_ER),
	.SFS_ER		(SFS_ER),
	.SUBOP_ER	(IR_ER[27:24]),

        .iFLAG_E	(iFLAG_E),
        .iCARRY_E	(iCARRY_E),
        .iOVERFL_E	(iOVERFL_E),

//* inputs from MAC
	.MULDIVOP	(),	//* multicycle : mul32s, mul32u
	.MULDIV_RES	(),
	.MULDIV_RES_ACK (),
	
	.CLK		(CLK),
	.RSTN		(RSTN_D2)
);

endmodule
