/////////////////////////////////////////////////////////////////////////////
// This file is a part of the OpenRISC 1601 Verilog-HDL model
// Copyright (C) 2000 Jimmy C. Chen and OPENCORES.ORG
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// See the file COPYING for the full details of the license.
//
// OPENCORES.ORG is looking for new open source IP cores and developers that
// would like to help in our mission.
////////////////////////////////////////////////////////////////////////////
// IP core: OpenRISC 1601, a 16-bit instruction set/32-bit datapath RISC 
//          processor core of OR1K family, according to OR16 ISA.
// Features:
//  * fast exception processing (see architecture manual for details)
//  * yet optimized for size (without MMU and Caches)
// 
// Documentation: http://www.opencores.org/cores/or1k/or1601/
////////////////////////////////////////////////////////////////////////////
// File: or1601isa.h
// Author: Jimmy Chen-Min Chen, jimmy@ee.nctu.edu.tw, 06/18/2000
// Description: definition of ISA binary encoding
//  
// Architecture Notes:
//  
//  
//  
// TO DO:
//
////////////////////////////////////////////////////////////////////////////
// Revision History
// $Log$
`define H_NOP			16'hCA04	

`define H_OP_LOAD		4'h0
`define H_SUBOP_LW     		4'h0
`define H_SUBOP_LHZ    		4'h0
`define H_SUBOP_LBZ    		4'h0
`define H_SUBA_LHZ		4'h8
`define H_SUBA_LBZ		4'h9

`define H_OP_STOR		4'h1
`define H_SUBOP_SW    		4'h1
`define H_SUBOP_SH    		4'h1
`define H_SUBOP_SB    		4'h1
`define H_SUBSUBOP_SH		4'h8
`define H_SUBSUBOP_SB		4'h9

`define H_OP_MOVIBLS		4'h2
`define H_OP_MOVIBZE		4'h3
`define H_OP_MOVIBSE		4'h4

`define H_OP_LOAD_STOR_PAI	4'h5
`define H_SUBOP_LWPAIN		4'h0
`define H_SUBOP_LHZPAIN		4'h1
`define H_SUBOP_LBZPAIN		4'h2
`define H_SUBOP_LHSPAIN		4'h3
`define H_SUBOP_LBSPAIN		4'h4

`define H_SUBOP_SWPAI  		4'h5
`define H_SUBOP_SHPAI  		4'h6
`define H_SUBOP_SBPAI  		4'h7
`define H_SUBOP_SWPAIN 		4'h8
`define H_SUBOP_SHPAIN 		4'h9
`define H_SUBOP_SBPAIN 		4'hA

`define H_OP_ADDIU		4'h6
`define H_OP_ADDIS		4'h7

`define H_OP_JMP		4'h8
`define H_OP_JAL		4'h9

`define H_OP_BNF		4'hA
`define H_OP_BF			4'hB

`define H_OP_SF_EXT             4'hC
`define H_SUBOP_SFEQ            4'h0
`define H_SUBOP_SFNE            4'h1
`define H_SUBOP_SFGTS           4'h2
`define H_SUBOP_SFGES           4'h3
`define H_SUBOP_SFLTS           4'h4
`define H_SUBOP_SFLES           4'h5
`define H_SUBOP_SFGTU           4'h6
`define H_SUBOP_SFGEU           4'h7
`define H_SUBOP_SFLTU           4'h8
`define H_SUBOP_SFLEU           4'h9

`define H_SUBOP_EXT_JALR_JR     4'hA
`define H_SUBSUBOP_JALR         4'h5
`define H_SUBSUBOP_JR           4'h6

`define H_SUBSUBOP_EXTHS        4'h0
`define H_SUBSUBOP_EXTHZ        4'h1
`define H_SUBSUBOP_EXTBS        4'h2
`define H_SUBSUBOP_EXTBZ        4'h3
`define H_SUBSUBOP_NOP          4'h4

`define H_SUBOP_MULS            4'hB
`define H_SUBOP_MULU            4'hC
`define H_SUBOP_DIVS            4'hD
`define H_SUBOP_DIVU            4'hE

`define H_OP_MOV_ALU_EXP       	4'hD
`define H_SUBOP_BRK		4'h0
`define H_SUBOP_SYS		4'h8
`define H_SUBOP_RFE		4'h9

`define H_SUBOP_MOV		4'h1
`define H_SUBOP_MSS		4'h2
`define H_SUBOP_MAS		4'h3
`define H_SUBOP_MODS		4'h5
`define H_SUBOP_MODU		4'h7
`define H_SUBOP_SWAPB		4'h4
`define H_SUBOP_SWAPH		4'h6
`define H_SUBOP_SHLA 		4'hA
`define H_SUBOP_SHRA 		4'hC
`define H_SUBOP_SHRL 		4'hE
`define H_SUBOP_ROR  		4'hB
`define H_SUBOP_ROL  		4'hD

`define H_OP_SHIFT_ALUI		4'hE

`define H_SUBOP_NOTU   		4'h0
`define H_SUBOP_NEGS   		4'h1
`define H_SUBOP_XOR    		4'h2
`define H_SUBOP_OR    		4'h3
`define H_SUBOP_AND    		4'h4

`define H_SUBOP_XORI   		4'h5
`define H_SUBOP_ORI   		4'h6
`define H_SUBOP_ANDI  		4'h7

`define H_SUBOP_MTSR  		4'h8
`define H_SUBOP_MFSR  		4'h9

`define H_SUBOP_SHLAI		4'b101x
`define H_SUBOP_SHLAI3		3'b101
`define H_SUBOP_SHRAI		4'b110x
`define H_SUBOP_SHRAI3		3'b110
`define H_SUBOP_SHRLI		4'b111x
`define H_SUBOP_SHRLI3		3'b111

`define H_OP_ADDSUB_TEST	4'hF

`define H_SUBOP_ADDS		4'h0
`define H_SUBOP_SUBS		4'h1
`define H_SUBOP_ADDU		4'h2
`define H_SUBOP_SUBU		4'h3
`define H_SUBOP_ADDWCS		4'h4
`define H_SUBOP_SUBWCS		4'h5
`define H_SUBOP_TEST		4'h6
`define H_SUBOP_TESTI		4'b100x
`define H_SUBOP_TESTI3		3'b100
`define H_SUBOP_CLR 		4'hC
`define H_SUBOP_CLRI		4'b101x
`define H_SUBOP_CLRI3		3'b101
`define H_SUBOP_SET 		4'hD
`define H_SUBOP_SETI		4'b111x
`define H_SUBOP_SETI3		3'b111
