/////////////////////////////////////////////////////////////////////////////
// This file is a part of the OpenRISC 1601 Verilog-HDL model
// Copyright (C) 2000 Jimmy C. Chen and OPENCORES.ORG
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// See the file COPYING for the full details of the license.
//
// OPENCORES.ORG is looking for new open source IP cores and developers that
// would like to help in our mission.
////////////////////////////////////////////////////////////////////////////
// IP core: OpenRISC 1601, a 16-bit instruction set/32-bit datapath RISC 
//          processor core of OR1K family, according to OR16 ISA.
// Features:
//  * fast exception processing (see architecture manual for details)
//  * yet optimized for size (without MMU and Caches)
// 
// Documentation: http://www.opencores.org/cores/or1k/or1601/
////////////////////////////////////////////////////////////////////////////
// File: regfile.v
// Author: Jimmy Chen-Min Chen, jimmy@ee.nctu.edu.tw, 06/18/2000
// Description: register file
//  
// Architecture Notes:
//  
//  
//  
// TO DO:
//
////////////////////////////////////////////////////////////////////////////
// Revision History
// $Log$
`include "pool/timescale.v"
`include "pool/sys.h"

module regfile(AO, AADR, AREN, BO, BADR, BREN, WEN, WADR, WDI, CLK);
	output [31:0]	AO, BO;
	input  [3:0]	AADR, BADR;
	input		AREN, BREN; //* read enable
	input		WEN;
	input  [3:0]	WADR;
	input  [31:0]	WDI;
	input		CLK;
	
	reg    [31:0]	RPOOL[1:15];
	reg    [31:0]	AO, BO;
	
	always @(AADR or BADR or AREN or BREN or 
		              RPOOL[1]  or RPOOL[2]  or RPOOL[3]  or RPOOL[4] or
		 RPOOL[5]  or RPOOL[6]  or RPOOL[7]  or RPOOL[8]  or RPOOL[9] or
		 RPOOL[10] or RPOOL[11] or RPOOL[12] or RPOOL[13] or RPOOL[15]
	) begin
		if(AREN && AADR == 4'b0) AO = 32'b0;
		else		 	 AO = RPOOL[AADR];
			
		if(BREN && BADR == 4'b0) BO = 32'b0;
		else		 	 BO = RPOOL[BADR];
	end

/*
	always @(posedge CLK) begin
		if(~WEN && WADR != 4'b0)
			RPOOL[WADR] <= #1 WDI;
	end
*/
	always @(negedge CLK) begin
		if(~WEN && WADR != 4'b0)
			RPOOL[WADR] <= #1 WDI;
	end

	integer i;

`ifdef TRANSLATE_ON
//* clear register file at reset
	always @(posedge top.RSTN) begin
		for(i=0; i<16; i=i+1)
			RPOOL[i] = 32'hxxxx_xxxx;
	end
	
`endif

endmodule
