/////////////////////////////////////////////////////////////////////////////
// This file is a part of the OpenRISC 1601 Verilog-HDL model
// Copyright (C) 2000 Jimmy C. Chen and OPENCORES.ORG
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// See the file COPYING for the full details of the license.
//
// OPENCORES.ORG is looking for new open source IP cores and developers that
// would like to help in our mission.
////////////////////////////////////////////////////////////////////////////
// IP core: OpenRISC 1601, a 16-bit instruction set/32-bit datapath RISC 
//          processor core of OR1K family, according to OR16 ISA.
// Features:
//  * fast exception processing (see architecture manual for details)
//  * yet optimized for size (without MMU and Caches)
// 
// Documentation: http://www.opencores.org/cores/or1k/or1601/
////////////////////////////////////////////////////////////////////////////
// File: sys.h
// Author: Jimmy Chen-Min Chen, jimmy@ee.nctu.edu.tw, 06/18/2000
// Description: system definition of OR1601
//  
// Architecture Notes:
//  
//  
//  
// TO DO:
//
////////////////////////////////////////////////////////////////////////////
// Revision History
// $Log$
`define UD 1
`define CLKCYC 19

`define TRANSLATE_ON  //* equivalent to synopsys translate_off

`define RST_VEC   	32'h0000_0100
`define BUS_ERROR_VEC	32'h0000_0200
`define DPG_FAULT_VEC	32'h0000_0300
`define IPG_FAULT_VEC	32'h0000_0400
`define INT_VEC   	32'h0000_0500
`define UNALIGN_VEC   	32'h0000_0600
`define ILLEGAL_VEC   	32'h0000_0700
`define SF0_INT_VEC	32'h0000_0800
`define SF1_INT_VEC	32'h0000_0900
`define SF2_INT_VEC	32'h0000_0A00
`define SYS_VEC   	32'h0000_0C00
`define BRK_VEC   	32'h0000_0D00
`define OV_VEC   	32'h0000_0E00
`define DIVZERO_VEC   	32'h0000_0F00

`define ASEL_LOAD_DI	8'b0100_0000
`define ASEL_RF		8'b0010_0000
`define ASEL_IMME_CNST  8'b0001_0000
`define ASEL_IALU	8'b0000_1000
`define ASEL_SPR	8'b0000_0100
`define ASEL_RF_WDI_M	8'b0000_0010
`define ASEL_RF_WDI_WR  8'b0000_0001

`define BSEL_LOAD_DI	8'b0100_0000
`define BSEL_RF		8'b0010_0000
`define BSEL_IMME_CNST  8'b0001_0000
`define BSEL_IALU	8'b0000_1000
`define BSEL_SPR	8'b0000_0100
`define BSEL_RF_WDI_M	8'b0000_0010
`define BSEL_RF_WDI_WR  8'b0000_0001

`define STSEL_RF   	8'b0010_0000
`define STSEL_RF_WDI_M	8'b0000_0010
`define STSEL_RF_WDI_WR 8'b0000_0001

//* write back selection
`define SEL_RFWDI_SRA 	4'b1000
`define SEL_RFWDI_SPR 	4'b0100
`define SEL_RFWDI_LSU	4'b0010
`define SEL_RFWDI_ALUMR 4'b0001

// SPRG definition
`define SPRG_SYS 	4'h0
`define SPRG_MAC	4'h5

`define SPRG_SYS_CCR 	12'h000
`define SPRG_SYS_MPR 	12'h001
`define SPRG_SYS_SR  	12'h002
`define SPRG_SYS_PAIR	12'h003

`define SPRG_SYS_EPCR	12'h010
`define SPRG_SYS_EEAR	12'h030

`define SPRG_MAC_MACLO	12'h001
`define SPRG_MAC_MACHI	12'h002

// BIU's NBI(Native Bus Interface)
`define NBISM_IDLE      4'b0000
`define NBISM_IRACC     4'b1000
`define NBISM_IRACCLAST 4'b1001
`define NBISM_DRACC     4'b0100
`define NBISM_DRACCLAST 4'b0101
`define NBISM_DWACC     4'b0010
`define NBISM_DWACCLAST 4'b0011

