/////////////////////////////////////////////////////////////////////////////
// This file is a part of the OpenRISC 1601 Verilog-HDL model
// Copyright (C) 2000 Jimmy C. Chen and OPENCORES.ORG
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// See the file COPYING for the full details of the license.
//
// OPENCORES.ORG is looking for new open source IP cores and developers that
// would like to help in our mission.
////////////////////////////////////////////////////////////////////////////
// IP core: OpenRISC 1601, a 16-bit instruction set/32-bit datapath RISC 
//          processor core of OR1K family, according to OR16 ISA.
// Features:
//  * fast exception processing (see architecture manual for details)
//  * yet optimized for size (without MMU and Caches)
// 
// Documentation: http://www.opencores.org/cores/or1k/or1601/
////////////////////////////////////////////////////////////////////////////
// File: topcpu.v
// Author: Jimmy Chen-Min Chen, jimmy@ee.nctu.edu.tw, 06/18/2000
// Description: top testbench.
//  
// Architecture Notes:
//  
//  
//  
// TO DO:
//
////////////////////////////////////////////////////////////////////////////
// Revision History
// $Log$
`include "pool/timescale.v"
`include "pool/sys.h"

module top;

	wire   [31:0]	NBI_ADDR;
	wire   [31:0]	NBI_DO, NBI_DI;
	wire   [1:0]	NBI_SIZE;

	reg    [2:0]	RSTN_CNT;
	reg		CLK, RSTN;
	reg		PASS1, FAIL1;
	wire		NBI_REQN, NBI_RW;
	wire		NBI_GNTN;
	
	initial begin
		FAIL1 = 1'b0;
		PASS1 = 1'b0;
		CLK = 1'b0;
		RSTN = 1'b0;
	#100	RSTN = 1'b1;
	end

always #(`CLKCYC*0.5) CLK = ~CLK;

`ifdef FSDB
        initial begin
                $fsdbDumpfile("topcpu.fsdb");
                $fsdbDumpvars(0, top);
        end
`endif

`define TESTING_PGM top.BRIDGE1.cur_filename

	always @(posedge CLK)
        if(~NBI_REQN && ~NBI_RW && ~NBI_GNTN) begin
                if(NBI_ADDR === 32'hffff_aaa0 && NBI_DO === 32'h0000abcd) begin
			PASS1 <= #(`UD) 1'b1;
                end
                if(NBI_ADDR === 32'hffff_aaa0 && NBI_DO === 32'h0000dead) begin
			FAIL1 <= #(`UD) 1'b1;
                end
        end
	always @(posedge CLK)
	if(~NBI_REQN && ~NBI_RW && ~NBI_GNTN) begin
	 	if(NBI_ADDR === 32'hffff_aaa4 && NBI_DO === 32'h0000abcd) begin
		  if(PASS1) begin
			$display("OOO %s PASS OOO", `TESTING_PGM);	
			RSTN <= #(`UD) 1'b0;
			PASS1 <= #(`UD) 1'b0;
			FAIL1 <= #(`UD) 1'b0;
		  end
		  else begin
			$display("XXX not PASS1 but %s FAIL OOO", `TESTING_PGM);
			RSTN <= #(`UD) 1'b0;
			PASS1 <= #(`UD) 1'b0;
			FAIL1 <= #(`UD) 1'b0;
			#100 $finish;
		  end
		end
	 	if(NBI_ADDR === 32'hffff_aaa4 && NBI_DO === 32'h0000dead) begin
		  if(FAIL1) begin
			$display("XXX %s FAIL OOO", `TESTING_PGM);
			RSTN <= #(`UD) 1'b0;
			PASS1 <= #(`UD) 1'b0;
			FAIL1 <= #(`UD) 1'b0;
			#100 $finish;
		  end
		  else begin
			$display("XXX not FAIL1 %s FAIL OOO", `TESTING_PGM);
			RSTN <= #(`UD) 1'b0;
			PASS1 <= #(`UD) 1'b0;
			FAIL1 <= #(`UD) 1'b0;
			#100 $finish;
		  end
		end
	end
	else if(RSTN == 1'b0 && RSTN_CNT == 3'b111) begin
		RSTN <= #(`UD) 1'b1;
	end

	always @(posedge CLK) begin
		if(RSTN == 1'b0) RSTN_CNT <= #(`UD) RSTN_CNT + 1'b1;
		else 		 RSTN_CNT <= #(`UD) 3'b0;
	end

wire [1:0] ASYNC_EXT_INTN = 2'b11;

reg EXT_INTN;
reg [7:0] INT_PERIOD;

always @(posedge CLK)
	if(RSTN == 1'b0)
		EXT_INTN <= #(`UD) 1'b1;
	else if(top.CPU1.IU1.SPRG0.INT_ENABLE_MR) begin
		if(INT_PERIOD == 0)
			EXT_INTN <= #(`UD) 1'b0;
		else if(INT_PERIOD == 10)
			EXT_INTN <= #(`UD) 1'b1;
	end
	else
		EXT_INTN <= #(`UD) 1'b1;

always @(posedge CLK)
	if(RSTN == 1'b0)
		INT_PERIOD <= #(`UD) 8'b0;
	else if(INT_PERIOD == 50)
		INT_PERIOD <= #(`UD) 8'b0;
	else if(~top.CPU1.iBIU2LSU_BUSY && ~top.CPU1.iBIU2INU_BUSY)
		INT_PERIOD <= #(`UD) INT_PERIOD + 1;

CPU CPU1(
// 
	.INTREQN	({EXT_INTN, ASYNC_EXT_INTN}),	// asynchronous external interrupt pins, low active

// OR1K native bus interface(NBI)
        .NBI_ADDR	(NBI_ADDR),               // addr out
        .NBI_DO		(NBI_DO),                 // data out
        .NBI_SIZE	(NBI_SIZE),               // 2 bit size
        .NBI_RW		(NBI_RW),                 // read(1)/write(0)
        .NBI_BURSTN	(NBI_BURSTN),             // burst transaction
        .NBI_REQN	(NBI_REQN),               // NBI request to bridge

        .NBI_DI		(NBI_DI),                 // data in
        .NBI_GNTN	(NBI_GNTN),               // bridge grants the request of NBI_REQN

// system
        .CLK		(CLK),
        .RSTN		(RSTN)
);

BRIDGE BRIDGE1(
        .NBI_ADDR	(NBI_ADDR),               // addr out
        .NBI_DO		(NBI_DO),                 // data out
        .NBI_SIZE	(NBI_SIZE),               // 2 bit size
        .NBI_RW		(NBI_RW),                 // read(1)/write(0)
        .NBI_BURSTN	(NBI_BURSTN),             // burst transaction
        .NBI_REQN	(NBI_REQN),               // NBI request to bridge

        .NBI_DI		(NBI_DI),                 // data in
        .NBI_GNTN	(NBI_GNTN),               // bridge grants the request of NBI_REQN

// system
        .CLK		(CLK),
        .RSTN		(RSTN)
);

/////////////////////////////////////////////////////////////////////

`include "pool/DEBUGREG_CPU.v"

endmodule
/*
        $Author$
        $Date$
        $Header$
        $Id$
        $Locker$
        $Name$
        $RCSfile$
        $Revision$
        $Source$
        $State$
        $Log$
 */

